/******************************************************************************
*******************************************************************************
**
**    Copyright 1995-1999 Regents of the University of Minnesota
**    Portions Copyright 1999 Sistina Software, Inc.  <kpreslan@sistina.com>
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Kenneth Preslan <kpreslan@lcse.umn.edu>
** 
**    Module name:  dio.h
**
**    Description:
**      This module implements a bunch of routines that do I/O through the
**        buffer cache.
**
*******************************************************************************
******************************************************************************/

#ifndef __DIO_DOT_H__
#define __DIO_DOT_H__


/*  Flags to gfs_dwrite  */

#define DW_NOKEEP    (0x00000010)
#define DW_NOTMETA   (0x00000040)  


/*  Buffer manipulation routines called by the glock routines  */

void gfs_inval_bh(gfs_glock_t *gl);
void gfs_sync_bh(gfs_glock_t *gl);

/*  Asynchronous I/O Routines  */

buffer_head_t *gfs_dgetblk(gfs_sbd_t *sdp, uint64 blkno, uint64 glock);
buffer_head_t *gfs_dread(gfs_sbd_t *sdp, uint64 blkno, uint64 glock);
int gfs_dreread(gfs_sbd_t *sdp, int len, buffer_head_t **bh, int wait);
int gfs_dwrite(buffer_head_t *bh, int flags);
int gfs_dforget(buffer_head_t *bh);

void gfs_mark_uptodate(buffer_head_t *bh, int state);
int gfs_buffer_uptodate(buffer_head_t *bh);
void gfs_wait_on_buffer(buffer_head_t *bh);

/*  Synchronous I/O routines  */

buffer_head_t *gfs_sread(gfs_sbd_t *sdp, uint64 bbno, uint32 bytes);
int gfs_swrite(buffer_head_t *bh);
int gfs_sforget(buffer_head_t *bh);

#endif  /*  __DIO_DOT_H__  */
