/******************************************************************************
*******************************************************************************
**
**    Copyright 1995-1999 Regents of the University of Minnesota
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Kenneth Preslan <kpreslan@lcse.umn.edu>
** 
**    Module name:  debug.h
**
**    Description:
**      This module implements dynamic debug messages in GFS
**
*******************************************************************************
******************************************************************************/

#ifndef __GFS_DEBUG_DOT_H__
#define __GFS_DEBUG_DOT_H__

#include "global.h"



#define DEBUG_HOW ""
/*#define DEBUG_HOW KERN_DEBUG*/



/* GFS Debugging Flags */

#define DEBUG_BITS1             (0x0000000000000001ll)
#define DEBUG_BITS2             (0x0000000000000002ll)
#define DEBUG_BITS3             (0x0000000000000004ll)

#define DEBUG_BLKLIST1          (0x0000000000000008ll)
#define DEBUG_BLKLIST2          (0x0000000000000010ll)
#define DEBUG_BLKLIST3          (0x0000000000000020ll)

#define DEBUG_BMAP1             (0x0000000000000040ll)
#define DEBUG_BMAP2             (0x0000000000000080ll)
#define DEBUG_BMAP3             (0x0000000000000100ll)

#define DEBUG_DCACHE1           (0x0000000000000200ll)
#define DEBUG_DCACHE2           (0x0000000000000400ll)
#define DEBUG_DCACHE3           (0x0000000000000800ll)

#define DEBUG_DIO1              (0x0000000000001000ll)
#define DEBUG_DIO2              (0x0000000000002000ll)
#define DEBUG_DIO3              (0x0000000000004000ll)

#define DEBUG_DIR1              (0x0000000000008000ll)
#define DEBUG_DIR2              (0x0000000000010000ll)
#define DEBUG_DIR3              (0x0000000000020000ll)

#define DEBUG_GLOCK1            (0x0000000000040000ll)
#define DEBUG_GLOCK2            (0x0000000000080000ll)
#define DEBUG_GLOCK3            (0x0000000000100000ll)

#define DEBUG_FILE1             (0x0000000000200000ll)
#define DEBUG_FILE2             (0x0000000000400000ll)
#define DEBUG_FILE3             (0x0000000000800000ll)

#define DEBUG_FS1               (0x0000000001000000ll)
#define DEBUG_FS2               (0x0000000002000000ll)
#define DEBUG_FS3               (0x0000000004000000ll)

#define DEBUG_INODE1            (0x0000000008000000ll)
#define DEBUG_INODE2            (0x0000000010000000ll)
#define DEBUG_INODE3            (0x0000000020000000ll)

#define DEBUG_ONDISK1           (0x0000000040000000ll)
#define DEBUG_ONDISK2           (0x0000000080000000ll)
#define DEBUG_ONDISK3           (0x0000000100000000ll)

#define DEBUG_SUPER1            (0x0000000200000000ll)
#define DEBUG_SUPER2            (0x0000000400000000ll)
#define DEBUG_SUPER3            (0x0000000800000000ll)

#define DEBUG_SEMAPHORE         (0x0000001000000000ll)

#define DEBUG_DEADLOCK          (0x0000002000000000ll)

#define DEBUG_RETRIES           (0x0000004000000000ll)

#define DEBUG_PARANOIA          (0x0000008000000000ll)

#define DEBUG_LOCKMODULE        (0x0000010000000000ll)



#ifdef __KERNEL__

#ifdef IFDEBUG
#undef IFDEBUG
#endif  /*  IFDEBUG  */

#ifdef DEBUG
#define IFDEBUG(flag, x1) {if ((GFS_dflags & flag)) {x1}}
#else
#define IFDEBUG(flag, x1)
#endif

EXTERN uint64 GFS_dflags INIT(0);

#else  /*  __KERNEL  */

const debugFlag_t GFS_DFLAGS[]=
{
  {"bits1",             DEBUG_BITS1},
  {"bits2",             DEBUG_BITS2},
  {"bits3",             DEBUG_BITS3},
  {"bits",              DEBUG_BITS1 | DEBUG_BITS2 | DEBUG_BITS3},

  {"blklist1",          DEBUG_BLKLIST1},
  {"blklist2",          DEBUG_BLKLIST2},
  {"blklist3",          DEBUG_BLKLIST3},
  {"blklist",           DEBUG_BLKLIST1 | DEBUG_BLKLIST2 | DEBUG_BLKLIST3},

  {"bmap1",             DEBUG_BMAP1},
  {"bmap2",             DEBUG_BMAP2},
  {"bmap3",             DEBUG_BMAP3},
  {"bmap",              DEBUG_BMAP1 | DEBUG_BMAP2 | DEBUG_BMAP3},

  {"dcache1",           DEBUG_DCACHE1},
  {"dcache2",           DEBUG_DCACHE2},
  {"dcache3",           DEBUG_DCACHE3},
  {"dcache",            DEBUG_DCACHE1 | DEBUG_DCACHE2 | DEBUG_DCACHE3},

  {"dio1",              DEBUG_DIO1},
  {"dio2",              DEBUG_DIO2},
  {"dio3",              DEBUG_DIO3},
  {"dio",               DEBUG_DIO1 | DEBUG_DIO2 | DEBUG_DIO3},

  {"dir1",              DEBUG_DIR1},
  {"dir2",              DEBUG_DIR2},
  {"dir3",              DEBUG_DIR3},
  {"dir",               DEBUG_DIR1 | DEBUG_DIR2 | DEBUG_DIR3},

  {"glock1",            DEBUG_GLOCK1},
  {"glock2",            DEBUG_GLOCK2},
  {"glock3",            DEBUG_GLOCK3},
  {"glock",             DEBUG_GLOCK1 | DEBUG_GLOCK2 | DEBUG_GLOCK3},

  {"file1",             DEBUG_FILE1},
  {"file2",             DEBUG_FILE2},
  {"file3",             DEBUG_FILE3},
  {"file",              DEBUG_FILE1 | DEBUG_FILE2 | DEBUG_FILE3},

  {"fs1",               DEBUG_FS1},
  {"fs2",               DEBUG_FS2},
  {"fs3",               DEBUG_FS3},
  {"fs",                DEBUG_FS1 | DEBUG_FS2 | DEBUG_FS3},

  {"inode1",            DEBUG_INODE1},
  {"inode2",            DEBUG_INODE2},
  {"inode3",            DEBUG_INODE3},
  {"inode",             DEBUG_INODE1 | DEBUG_INODE2 | DEBUG_INODE3},

  {"ondisk1",           DEBUG_ONDISK1},
  {"ondisk2",           DEBUG_ONDISK2},
  {"ondisk3",           DEBUG_ONDISK3},
  {"ondisk",            DEBUG_ONDISK1 | DEBUG_ONDISK2 | DEBUG_ONDISK3},

  {"super1",            DEBUG_SUPER1},
  {"super2",            DEBUG_SUPER2},
  {"super3",            DEBUG_SUPER3},
  {"super",             DEBUG_SUPER1 | DEBUG_SUPER2 | DEBUG_SUPER3},

  {"semaphore",         DEBUG_SEMAPHORE},

  {"deadlock",          DEBUG_DEADLOCK},

  {"retries",           DEBUG_RETRIES},

  {"paranoia",          DEBUG_PARANOIA},

  {"lockmodule",        DEBUG_LOCKMODULE},

  {"all",		0xFFFFFFFFFFFFFFFFll},
  {"none",		0x0},

  {"",                  0x0}  /*  This must be the last flag.  */
};

#endif  /*  __KERNEL__  */



#endif  /*  __GFS_DEBUG_DOT_H__  */
