/******************************************************************************
*******************************************************************************
**
**    Copyright 1995-1999 Regents of the University of Minnesota
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Kenneth W. Preslan <kpreslan@lcse.umn.edu>
** 
**    Module name:  gfs_ioctl.h
**
**    Description:  The GFS ioctl()s
**
*******************************************************************************
******************************************************************************/

#ifndef __GFS_IOCTL_DOT_H__
#define __GFS_IOCTL_DOT_H__

#define _GFSC_(x)		(('G' << 8) | x)

/* Ioctls implemented */
#define GFS_DEBUG_ON            _GFSC_(0x00000001)
#define GFS_DEBUG_OFF           _GFSC_(0x00000002)
#define GFS_DUMPSTATS           _GFSC_(0x00000003)
#define GFS_SHOW_SUBPOOLS       _GFSC_(0x00000004)
#define GFS_WHICH_SUBPOOL       _GFSC_(0x00000005)
#define GFS_MOVE_TO_SUBPOOL  	_GFSC_(0x00000006)

#define GFS_MAXPATHLEN		256
#define MB 			(1 << 20)



/* --- Structures ----- */


/*
 * Structures used by the GFS_SHOW_SUBPOOLS ioctl
 */

/* Status of each subpool */
typedef struct gfs_spstat_s
{
  uint64 	bfree;
  uint64 	blocks;
  uint64 	dinds;
} gfs_spstat_t;



/* List of the status of subpools */
typedef struct gfs_spstatlist_s
{
  uint32 	sp_cnt;    /*  Number of subpools in pool  */
  gfs_spstat_t  *sp_list;  /*  Array of subpool info structs  */
} gfs_spstatlist_t;




/* 
 *Structure used by the GFS_WHICH_SUBPOOL ioctl 
 */

/* Number of blocks belonging to a file in a subpool */
typedef struct gfs_fspstatlist_s
{
  uint32 	fsp_cnt;   /* No. of sp's having file blocks */
  uint64 	*blks;     /* No. of blocks in each sp */
} gfs_fspstatlist_t;




/* 
 * Structures used by the GFS_MOVE_TO_SUBPOOL ioctl 
 */

typedef struct gfs_lnkname_s
{
  char  	dname[GFS_MAXPATHLEN]; /* directory containing the hardlink */
  char 		fname[GFS_MAXPATHLEN]; /* the name of the hardlink */
  unsigned int 	dlen;		       /* length of dname entry */
  unsigned int  flen;		       /* length of fname entry */
} gfs_lnkname_t;

typedef struct 	gfs_lnkinfo_s
{
  uint32 	d_sp;	   /* Destination Subpool */ 
  uint32 	lnk_cnt;   /* Number of hardlinks to the file */ 
  gfs_lnkname_t	*l_name;   /* Names of the hardlinks */
} gfs_lnkinfo_t;



#endif  /*  __GFS_IOCTL_DOT_H__  */
