/******************************************************************************
*******************************************************************************
**
**    Copyright 1995-1999 Regents of the University of Minnesota
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Seth Van Oort <vano0023@tc.umn.edu>
** 
**    Module name: gfs_profile.h
**
*******************************************************************************
******************************************************************************/


#ifndef __GFS_DOT_PROFILE_H__
#define __GFS_DOT_PROFILE_H__

#include "global.h"
#include "incore.h"

struct gfs_sbd_s;

int gfs_init_tunevals(struct gfs_sbd_s *sbd, int allocate);

typedef struct gfs_profile {
  uint64 total_calls;
  uint64 total_micros;
} gfs_profile_t;

typedef struct gfs_tune_pr {

  /* max % full in leaf before reading in adjacent leaf to try a merge */

  uint8   pr_try_leaf_mrg; 

  /* max % full that merged leaf will have, before doing a merge */

  uint8   pr_do_leaf_mrg;  

} gfs_tune_pr_t;

typedef struct gfs_tune {
  struct gfs_tune_pr pr;

  uint32 b_try_leaf_mrg;
  uint32 b_do_leaf_mrg;
} gfs_tune_t;


extern char *fn_names[];
extern gfs_tune_t tune;
/*extern gfs_tune_vals_t tune_vals;*/

#ifdef DEBUG
extern int stack[100];
extern int stack_ct;
#endif

#ifdef DEBUG
#define PRINT_ENTER(x, flag) \
{ stack[stack_ct] = x; \
  stack_ct++;  \
               \
  if ((GFS_dflags & flag)) \
{printk("Entering %s\n", fn_names[x] );}} 
#else
#define PRINT_ENTER(x, flag)
#endif

#ifdef PROFILE
#define PROFILE_ENTER(x) \
{gfs_prof[x].total_calls++; \
 do_gettimeofday(&gfs_tv); \
 gfs_prof[x].total_micros -= (uint64)(gfs_tv.tv_usec + (uint64)(1000000 * gfs_tv.tv_sec));}
#else
#define PROFILE_ENTER(x)
#endif

#define ENTER(x, flag) \
{PRINT_ENTER(x, flag)  \
   PROFILE_ENTER(x)}

#ifdef DEBUG
#define PRINT_EXIT(x, flag) \
{stack_ct--; \
              \
 if ((GFS_dflags & flag)) \
{printk("Exiting %s\n", fn_names[x] );}} 
#else
#define PRINT_EXIT(x, flag)
#endif

#ifdef PROFILE
#define PROFILE_EXIT(x) \
{gfs_prof[x].total_calls++; \
 do_gettimeofday(&gfs_tv); \
 gfs_prof[x].total_micros += (uint64)(gfs_tv.tv_usec + (uint64)(1000000 * gfs_tv.tv_sec));}
#else
#define PROFILE_EXIT(x)
#endif

#define EXIT(x, flag) \
{PRINT_EXIT(x, flag) \
PROFILE_EXIT(x)}

enum {     GFS_DIRENT_FIRST, /* dir.c */
	   GFS_DIRENT_NEXT,
	   GFS_DIRENT_DEL,
	   GFS_DIRENT_ALLOC,
	   LEAF_SEARCH,
	   GET_LEAF,
	   GET_LEAF_NR,
	   GET_FIRST_LEAF,
	   GET_NEXT_LEAF,
	   LINKED_LEAF_SEARCH,
	   LEAF_PACK,
	   LEAF_MERGE,
	   GET_ADJ_LEAFSIZE,
	   DIR_HALVE_EXHASH,
	   CHECK_LEAF_MERGE,
	   DIR_MAKE_EXHASH,
	   DIR_SPLIT_LEAF,
	   DIR_DOUBLE_EXHASH,
	   DIR_E_SEARCH,
	   DIR_E_ADD,
	   DIR_E_DEL,
	   DO_FILLDIR,
	   DIR_E_TRANS,
	   DIR_E_MVINO,
	   DIR_L_SEARCH,
	   DIR_L_ADD,
	   DIR_L_DEL,
	   DIR_L_TRANS,
	   DIR_L_MVINO,
	   DIR_STATS,
	   GFS_DIR_SEARCH,
	   GFS_DIR_ADD,
	   GFS_DIR_DEL,
	   GFS_DIR_TRANS,
	   GFS_DIR_MVINO,
	   GFS_DIR_FREE,
	   GFS_INO2GL, /* fs.c */
	   GFS_IHASHLOOKUP,
	   GFS_IHASHADD,
	   GFS_IHASHDEL,
	   INDIRFREE,
	   FREEFILE,
	   GFS_DIFREE,
	   GFS_RANDOM,
	   GFS_RANDRANGE,
	   GFS_CHANGE_NLINK,
	   GFS_TRUNCATE,
	   GFS_INODE_ATTR_IN, /* inode.c */
	   GFS_INODE_ATTR_OUT,
	   GFS_LOOKUPI,
	   GFS_LOOKUP,
	   GFS_CREATEI,
	   GFS_CREATE,
	   GFS_LINK,
	   UNLINKI,
	   GFS_UNLINK,
	   GFS_SYMLINK,
	   GFS_MKDIR,
	   RMDIRI,
	   GFS_RMDIR,
	   GFS_MKNOD,
	   GFS_IS_SUBDIR,
	   GFS_RENAME,
	   GFS_READLINK,
	   GFS_IREVALIDATE,
	   GFS_FOLLOW_LINK,
	   GFS_BMAP,
 	   NR_FUNCTIONS
};
	     
#ifdef PROFILE
extern struct gfs_profile gfs_prof[NR_FUNCTIONS];
extern struct timeval gfs_tv;


#endif	     



#endif
