/******************************************************************************
*
*    Copyright 1999 Regents of the University of Minnesota
*
*    This is free software released under the GNU General Public License.
*    There is no warranty for this software.  See the file COPYING for
*    details.
*
*    See the file CONTRIBUTORS for a list of contributors.
*
*    This file is maintained by:
*      Mike C Tilstra <tilstra@lcse.umn.edu>
* 
*    Module name: dlock_ip.o
*/
#ifndef __dlrpc_h__
#define __dlrpc_h__

#include <asm/types.h>
#include <linux/net.h>
#include <linux/in.h>
#include <linux/inet.h> /* in_aton */
#include <linux/socket.h>
#include <linux/sunrpc/clnt.h>

/* minimum macro */
#ifndef MIN
#define MIN(a,b) ((a<b)?a:b)
#endif

/* The dlock structures with which we do our work.
 * WARNING!!!!! These look different from the ones that the server sees!!!
 *   This is because we need to know how big some pointers are before we
 *   try to fill them. (so we don't overfill)
 *  This is the moving memory to&from user space.  We need to have created
 *  the space in the user process beforehand, and inform the inside how big
 *  it is.  Thus `dsr' == desired.  Those fields say how big the pointer
 *  is.  Then the minimum of the actual and the dsr is copied into the
 *  pointer.
 */
/******* First the CDBS ***********/
struct dlock_CDB{
	u8  opcode;
	u8  action;   
	u32 lock_number;
	u8  input_version_number;
	u32 allocation_length;
	u32 cid;
};

struct mode_CDB {
	u8  tag;
	u8  max_clients;
	u32 num_locks;
	u32 lock_time_ms;
};

struct CDBs {
	int  type;
	union {
		struct dlock_CDB dlock; /*Type==0*/
		struct mode_CDB mode;   /*Type==1*/
	} cdb_u;
};
typedef struct CDBs CDBs;

/******** Then the RDFs ********/
/* NOTICE: that these are a bit different than the ones from in the server!
 */
struct dlock_RDF{
	u8  result;
	u8  act;
	u8  expired;
	u8  state;
	u32 version_number;
	u8  num_holders;
	u16 dsr_cnt; /* This item is *NOT* passed over the network */
	u32 *cids;
};

struct rexpired_RDF{
	u16 return_len;
	u16 dsr_dlen; /* This item is *NOT* passed over the network */
	u8  *exlock_block;
};

struct mode_page_RDF{
	u32 num_locks;
	u8  max_clients;
	u32 lock_time_ms;
};

struct RDFs {
	int  type;
	union {
		struct dlock_RDF dlock;    /* Type==0 */
		struct rexpired_RDF rex;   /* Type==1 */
		struct mode_page_RDF mpag; /* Type==2 */
	} rdf_u;
};
typedef struct RDFs RDFs;

/* prototypes */
int dlip_create_client(unsigned long addr, struct rpc_clnt **client);
void setup_rpc_stats(void);
void shutdown_rpc_stats(void);

/* RPC stuff that is quite needed */
#define DLIP_PROGRAM ((u_long)0x3A042635)
#define DLIP_VERSION ((u_long)1)
#define DLIP_NULL ((u_long)0)
#define DLIP_DODLOCK ((u_long)1)


#endif /* dlrpc.h */
