/******************************************************************************
*
*    Copyright 1999 Regents of the University of Minnesota
*
*    This is free software released under the GNU General Public License.
*    There is no warranty for this software.  See the file COPYING for
*    details.
*
*    See the file CONTRIBUTORS for a list of contributors.
*
*    This file is maintained by:
*      Mike C Tilstra <tilstra@lcse.umn.edu>
* 
*    Module name: dlock_tcp.o
*
* All data packets are transmitted as knownlength chunks.  Since the server
* will be informing clients when to use callbacks, cid lists are nolonger
* returned.
*
* The first field in every datapacket is a code field.  This field is used
* to determin the contents of the packet.
*/
#ifndef __dlip_transp_h__
#define __dlip_transp_h__

#ifndef __KERNEL__
typedef unsigned char u8;
typedef unsigned int  u16;
typedef unsigned long u32;
#endif

/* Values for the code field */
#define CodeLess 0x00
#define CDBdlock 0x01
#define RDFdlock 0x11
#define RDFexprd 0x13
#define CDBmoder 0x04
#define CDBmodew 0x08
#define RDFmode  0x14
#define CallBack 0x20

/* Masks for types above */
#define COServCmd 0x20
#define CORDF     0x10

/******* First the CDBS ***********/
typedef struct dlock_CDB{
	u8  code;
	u8  action;   
	u8  ivn;
	u32 locknum;
	u32 cid;
} dlock_cdb_t;

typedef struct mode_page {
	u8  code;
	u8  max_clients;
	u32 num_locks;
	u32 lock_time_ms;
} mode_page_t;

/******** Then the RDFs ********/
typedef struct dlock_RDF{
	u8  code;
	u8  result;
	u8  act;
	u8  expired;
	u8  state;
	u8  num_holders;
	u32 version_number;
} dlock_rdf_t;

struct rexpired_RDF{ /* ignoring this for a while */
	u8  code;
	u16 return_len;
	u8  *exlock_block;
};

typedef struct drop_RDF {
  u8    code;
  u32   lock;
  u32   flags;
} drop_rdf_t;

#endif /* __dlip_transp_h__ */
