/******************************************************************************
*
*    Copyright 1999 Regents of the University of Minnesota
*
*    This is free software released under the GNU General Public License.
*    There is no warranty for this software.  See the file COPYING for
*    details.
*
*    See the file CONTRIBUTORS for a list of contributors.
*
*    This file is maintained by:
*      Mike C Tilstra <tilstra@lcse.umn.edu>
* 
*    Module name: dlock_tcp.o
*
*
*  eraser is a another thread that will run the gathered tasks from a
*  queue.  Its current need is for callback execution, but there may be
*  other things in the future.
*
*  The reason for this thread instead of one of the existing task queues is
*  that we need to be able to do io and kmallocs and all of that stuff that
*  you cannot do in the other queues.
*
*  So why is it eraser?  Because I am in a particularly bad mood and that
*  happens to be the song I'm listening to.  Die DIE dIE diE 
*/
#include <linux/kernel.h>
#include <linux/slab.h>
#include <linux/fs.h>
#include <linux/smp_lock.h>
#define __KERNEL_SYSCALLS__
#include <linux/unistd.h>

#include "dlip.h"

static struct semaphore eraser_mux = MUTEX_LOCKED;
static int eraser_intact = FALSE;
struct wait_queue *eraser_wait;
static struct semaphore eraser_p = MUTEX_LOCKED;

DECLARE_TASK_QUEUE(tq_eraser);

int eraser(void *args) 
{
	lock_kernel();

	exit_mm(current);

#if 0
	spin_lock_irq(&current->sigmask_lock);
	siginitsetinv(&current->blocked, sigmask(SIGKILL));
	recalc_sigpending(current);
	spin_unlock_irq(&current->sigmask_lock);
#endif

	current->session = 1;
	current->pgrp = 1;
	current->fs->umask = 0;

	sprintf(current->comm, "dlip_eraser");

	unlock_kernel();
	/*  Wake up the process who started us  */
	up(&eraser_mux);

	while (eraser_intact) {
		down(&eraser_p);
		run_task_queue(&tq_eraser);
	}

	up(&eraser_mux);
	printk("dlock_tcp: eraser ceasing to exist.\n");
	return 0;
}

void eraser_queue(struct tq_struct *tq)
{
	queue_task(tq, &tq_eraser);
	up(&eraser_p);
}

int create_eraser(void)
{
	int err;
	if(eraser_intact == FALSE) {
		eraser_intact = TRUE;
		err = kernel_thread(eraser, NULL, 0);
		if(err<0) {
			printe("Unable to create eraser (%d)\n", err);
			return err;
		}
		down(&eraser_mux);
	}
	return 0;
}

void destroy_eraser(void)
{
	eraser_intact = FALSE;
	up(&eraser_p);
	down(&eraser_mux);
}
