/******************************************************************************
*******************************************************************************
**
**    Copyright 1999 Sistina Software, Inc.  <kpreslan@sistina.com>
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Kenneth W. Preslan <kpreslan@lcse.umn.edu>
** 
**    Module name: main.c
**
*******************************************************************************
******************************************************************************/

#define __NO_VERSION__ 
#include <linux/module.h>
#include <linux/version.h>
char kernel_version [] = UTS_RELEASE;

#include <linux/kernel.h>
#include <linux/slab.h>

#include "gfs_locking.h"



typedef struct nolock_s
{
  char table_name[GFS_LOCKNAME_LEN + 1];

  gfs_callback_t *cb;
  void *fsdata;

  struct nolock_s *next;
} nolock_t;



nolock_t *list = NULL;



/******************************************************************************
*******************************************************************************
**
** nolock_mount()
**
** Description:
**
** Input(s):
**
** Output(s):
**
** Returns:
**
*******************************************************************************
******************************************************************************/

void *nolock_mount(char *table_name, char *host_data, gfs_callback_t *cb, void *fsdata)
{
  nolock_t *moddata;


  MOD_INC_USE_COUNT;



  for (moddata = list; moddata; moddata = moddata->next)
    if (!strcmp(moddata->table_name, table_name))
    {
      printk("Exiting nolock_mount() = table_exists\n");
      MOD_DEC_USE_COUNT;
      return(NULL);
    }


  moddata = (nolock_t *)kmalloc(sizeof(nolock_t), GFP_KERNEL);
  if (!moddata)
  {
    printk("Exiting nolock_mount() = out of memory\n");
    MOD_DEC_USE_COUNT;
    return(NULL);
  }
  memset(moddata, 0, sizeof(nolock_t));


  strcpy(moddata->table_name, table_name);

  moddata->cb = cb;
  moddata->fsdata = fsdata;


  moddata->next = list;
  list = moddata;



  return(moddata);
}


/******************************************************************************
*******************************************************************************
**
** nolock_unmount()
**
** Description:
**
** Input(s):
**
** Output(s):
**
** Returns:
**
*******************************************************************************
******************************************************************************/

void nolock_unmount(void *moddata)
{
  nolock_t *tmp, *last = NULL;
  int done = FALSE;


  if (list == moddata)
  {
    list = list->next;
    done = TRUE;
  }
  else
    for (tmp = list; tmp; last = tmp, tmp = tmp->next)
      if (tmp == moddata)
      {
	last->next = tmp->next;
	done = TRUE;
      }


  if (!done)
    printk("Given a weird moddata pointer\n");
  else
  {
    kfree(moddata);
    MOD_DEC_USE_COUNT;
  }
}


/******************************************************************************
*******************************************************************************
**
** nolock_lock()
**
** Description:
**
** Input(s):
**
** Output(s):
**
** Returns:
**
*******************************************************************************
******************************************************************************/

int nolock_lock(void *moddata, uint64 lock, int op, int flags, void **lockdata)
{
  return((LCKOUT_HELD) | (LCKOUT_CACHEABLE));
}


/******************************************************************************
*******************************************************************************
**
** nolock_unlock()
**
** Description:
**
** Input(s):
**
** Output(s):
**
** Returns:
**
*******************************************************************************
******************************************************************************/

int nolock_unlock(void *moddata, uint64 lock, int flags, void **lockdata)
{
  return(0);
}


gfs_lock_proto_t nolock_lock_proto =
{
  "nolock",
  nolock_mount,
  nolock_unmount,
  nolock_lock,
  nolock_unlock,
  NULL,  /*  release  */
  NULL
};


/******************************************************************************
*******************************************************************************
**
** init_module()
**
** Description:
**
** Input(s):
**
** Output(s):
**
** Returns:
**
*******************************************************************************
******************************************************************************/

int init_module(void)
{
  int error;

  if ((error = register_lock_proto(&nolock_lock_proto)) < 0)
    return(error);

  return(0);
}


/******************************************************************************
*******************************************************************************
**
** cleanup_module()
**
** Description:
**
** Input(s):
**
** Output(s):
**
** Returns:
**
*******************************************************************************
******************************************************************************/

void cleanup_module(void)
{
  unregister_lock_proto(&nolock_lock_proto);
}



/*  Symbols  */
EXPORT_NO_SYMBOLS;


