/******************************************************************************
*******************************************************************************
**
**    Copyright 1995-1999 Regents of the University of Minnesota
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      David C. Teigland  <teigland@lcse.umn.edu>
** 
**    Module name: pool_common.c
**
**    Description:
**	General, non-OS specific pool driver routines.  
**
*******************************************************************************
******************************************************************************/

#if 0
#include <linux/types.h>
#include <linux/fs.h>
#include <linux/blk.h>
#include <linux/malloc.h>
#include <linux/vmalloc.h>
#include <asm/uaccess.h>
#endif

#include <sys/param.h>

#include <sys/systm.h>
#include <sys/kernel.h>

#include <unistd.h>
#include <errno.h>

#include "pool_fbsd.h"
#include "pool.h"

#define printk printf
pool_t	pools[POOL_MAX_DEVICES];



/******************************************************************************
*******************************************************************************
** NAME
**	open_devices()
**
** DESCRIPTION
**	open all lower level devices including device lock devices.
**
** RETURN VALUE
**	Unix errno
**
*******************************************************************************
******************************************************************************/

int open_devices(pool_t *pp) 
{
  subpool_t           	*sp;
  device_t            	*devp;
  /*  dlockdev_t		*dlockdevp; */
  kdev_t 		dev;
  int			i,j, size;
  int			error = 0;


  IFDEBUG(POOL_DEBUG2,
	  printk("Entering open_devices()\n"););


  /* clear the inode and file pointers so we know what to back out */
  for (j=0; j<pp->p_num_spools; j++) {
    sp = &(pp->p_spools[j]);
    for (i=0; i<sp->sp_devs; i++) {
      devp = &(sp->sp_devlist[i]);
	  devp->vp = NULL;
#if 0
      devp->inode = NULL;
      devp->file = NULL;
#endif
    }
#if 0
    for (i=0; i<sp->sp_dldevs; i++) {
      dlockdevp = &(sp->sp_dldevlist[i]);
      dlockdevp->inode = NULL;
      dlockdevp->file = NULL;
    }
#endif
  }

  for (j=0; j < pp->p_num_spools; j++) {

    sp = &(pp->p_spools[j]);
	printf("open_devices sp: 0x%x\n",sp);
    for (i=0; i < sp->sp_devs; i++) {
      devp = &(sp->sp_devlist[i]);
      dev = devp->d_dev;
	  printf("open_devices devp: 0x%x\n",devp);
      IFDEBUG(POOL_DEBUG_OPEN,
			  printk("  open_devices: i:%d j:%d opening %d,%d (%d)\n",i,j,
					 MAJOR(dev), MINOR(dev), dev););
	  
      error = bdopen(devp, curproc);
	  
      if (error) {
		printk("pool: open failed: error %d opening %d,%d (%d)\n",
			   error, MAJOR(dev), MINOR(dev), dev);
		i--;	/* start closes with previous device */
		goto disk_backout;
      }
    }
  }
  
#if 0
  for (j=0; j<pp->p_num_spools; j++) {
    sp = &(pp->p_spools[j]);
    for (i=0; i<sp->sp_dldevs; i++) {
#if 0
      dlockdevp = &(sp->sp_dldevlist[i]);
      dev = dlockdevp->dl_dev;
#endif
      IFDEBUG(POOL_DEBUG_OPEN,
	      printk("  open_devices: opening %d,%d (%d)\n",
		     MAJOR(dev), MINOR(dev), dev););
#if 0
      error = bdopen(dev, &dlockdevp->inode, &dlockdevp->file);
#endif
      
      if (error) {
	printk("pool: open_devices: error bdopen\n");
	i--; 	/* start close... yada yada yada... */
	goto lock_backout;
      }
    }
  }
#endif

  /* Initialize the lock table */

#if 0
  if (pp->p_locktab == NULL) {

    size = pp->p_dlocks * sizeof(dlock_t);
    if (size > 127*1024)
      pp->p_locktab = (dlock_t *)vmalloc(size);
    else
      pp->p_locktab = (dlock_t *)kmem_alloc(size, KM_SLEEP);

    memset(pp->p_locktab, 0, size);
  }
#endif
  return (0);


lock_backout:
  for (; j >= 0; j--) {
    sp = &(pp->p_spools[j]);
    for (i = i > 0 ? i : sp->sp_dldevs - 1; i >= 0; i--) {
#if 0
      dlockdevp = &(sp->sp_dldevlist[i]);
      bdclose(&dlockdevp->inode, &dlockdevp->file);
#endif

    }
  }
  j = pp->p_num_spools - 1;
  i = 0;

disk_backout:
  for (; j >= 0; j--) {
    sp = &(pp->p_spools[j]);
    for (i = i > 0 ? i : sp->sp_devs - 1; i >= 0; i--) {
      devp = &(sp->sp_devlist[i]);
      bdclose(devp);
    }
  }
  return (error);
}


/******************************************************************************
*******************************************************************************
** NAME
**	close_devices
**
** DESCRIPTION
**	Close all lower level devices opened for this pool_t structure
**
** RETURN VALUE
**	Standard unix errno values
**
*******************************************************************************
******************************************************************************/

int close_devices(pool_t *pp) 
{
  subpool_t           	*sp;

  device_t           	*devp;
  int                 	i,j, size;
  int			error;
  kdev_t 		dev;

  IFDEBUG(POOL_DEBUG2,
	  printk("Entering close_devices()\n"););


  
  /* Close all storage and dlock devices */
  
  for (j=0; j<pp->p_num_spools; j++) {
    sp = &(pp->p_spools[j]);
    for (i=0; i<sp->sp_devs; i++) {
      devp = &(sp->sp_devlist[i]);
      dev = devp->d_dev;
#if 0
      fsync_dev(dev);
#endif
	  /*      IFDEBUG(POOL_DEBUG_OPEN, */
	  printk("  close_devices: closing %d,%d (%d)\n",
			 MAJOR(dev), MINOR(dev), dev);
	  /* ); */
	  error = bdclose(devp);
      if (error) {
		printk("close_devices: error %d\n", error);
      }
    }
  }
  
  return (0);
}


/******************************************************************************
*******************************************************************************
** NAME
**	print_pool()
**
** DESCRIPTION:  print labels for debugging
**
** RETURN VALUE
**
*******************************************************************************
******************************************************************************/

#ifdef DEBUG
int print_pool(pool_t *pp)
{
  subpool_t           	*spoolp;
  device_t            	*devp;
  int                 	i,j;

  printk("\nPool:\n");

  printk("name		 = %s\n", pp->p_name);
  printk("id             = %x\n", pp->p_pool_id);
  printk("version     	 = v%u.%u.%u\n", pp->p_version >> 16,
	 (pp->p_version >> 8) & 0xFF, pp->p_version & 0xFF);
  printk("subpools   	 = %u\n", pp->p_num_spools);
  printk("dlocks     	 = %u\n", pp->p_dlocks);
  printk("capacity   	 = %d\n", pp->p_capacity);
  printk("oblk        	 = %u\n", pp->p_oblk);
  printk("flags       	 = 0x%x\n", pp->p_flags);
  printk("\n");

  for (j=0; j < pp->p_num_spools; j++) {
    spoolp = &(pp->p_spools[j]);

    printk("Subpool %d:\n", j);
    printk("    sp_start     = %d\n", spoolp->sp_start);
    printk("    sp_len       = %u\n", spoolp->sp_len);
    printk("    sp_stripe    = %u\n", spoolp->sp_stripe);
    printk("    sp_devs      = %u\n", spoolp->sp_devs);
    printk("    sp_dlstart   = %d\n", spoolp->sp_dlstart);
	/*    printk("    sp_dlocks    = %u\n", spoolp->sp_dlocks); */
    printk("    sp_dldevs    = %u\n", spoolp->sp_dldevs);
    printk("\n");

    for (i=0; i < spoolp->sp_devs; i++) {
      devp = &(spoolp->sp_devlist[i]);
      printk("    Device %d:\n", i);
      printk("        d_dev   = (%d,%d)\n", GMAJOR(devp->d_dev),
	     GMINOR(devp->d_dev));
	  printk("     devicename = %s\n",devp->devicename);
      printk("        d_start = %d\n",devp->d_start);
      printk("        d_nblks = %d\n",devp->d_nblks);
    }
    printk("\n");
  }

  return(0);
}
#endif


/*****************************************************************************
 *****************************************************************************
 **
 ** int gen_pool()
 **
 ** Description:
 **   This routine takes a pointer to a pool_add_t structure and a pool_t
 **   structure and copies the data from the pointers to user space in the add
 **   struct to the pool struct. 
 **
 ** Input(s): 
 **   pointer to add structure.  This structure must be in kernel
 **   space, but ** the pointers in the add structure may be in user or kernel
 **   space
 **
 ** Output(s):
 **   a filled in pool_t structure
 **
 ** Returns:
 **   EFAULT, EINVAL, or 0 upon success
 **
 *****************************************************************************
 *****************************************************************************/

static int gen_pool(pool_add_t *add, pool_t *pp)
{
  subpool_t	*sp = NULL;
  device_t    	*nd = NULL;
#if 0
     dlockdev_t	*nl = NULL;
#endif
  int		i,j,k;
  int		size;

  IFDEBUG(POOL_DEBUG_ADD,
	  printk("Entered gen_pool()\n"););


  /* Perform a few simple tests.  Most error checking should be done by 
   * passemble.  Here we just check the pool version.
   * The on-disk pool label must match down to the subrevison.  Thus, versions
   * 0.3.0 and 0.4.0 don't match and are assumed to have different on disk
   * structures.   On the other hand opening a 0.4.2 label with a 0.4.1 or
   * 0.4.5 kernel is not an error */

  if ((add->pa_version >> 16) != (POOL_VERSION >> 16) || 
      ((add->pa_version >> 8) & 0xFF) != ((POOL_VERSION >> 8) & 0xFF)) {
    printk("Version mismatch in pool: %s:\n", add->pa_name);
    printk("\ton disk: v%d.%d.%d\n", add->pa_version >> 16,
	   (add->pa_version >> 8) & 0xFF, add->pa_version & 0xFF);
    printk("\tin core: v%d.%d.%d\n", POOL_VERSION >> 16,
	   (POOL_VERSION >> 8) & 0xFF, POOL_VERSION & 0xFF);
    return KRET(EINVAL);
  }

  pp->p_pool_id = add->pa_pool_id;
  pp->p_version = add->pa_version;
  pp->p_capacity = add->pa_capacity;
  pp->p_dlocks = add->pa_dlocks;
  pp->p_num_spools = add->pa_num_spools;
  memcpy(pp->p_name, add->pa_name, POOL_NAME_SIZE);

  
  /* copy data from user add structure to subpool_t */

  size = pp->p_num_spools * sizeof(subpool_t);

  pp->p_spools = (subpool_t *)kmem_alloc(size, KM_SLEEP);

  if (copyin(add->pa_spools, pp->p_spools, size)){	
    kmem_free(pp->p_spools, size);
    pp->p_spools = NULL;
    return KRET(EFAULT);
  }
  
  
  /* fill in the devices that constitute the subpools */

  for (k=0; k < pp->p_num_spools; k++) {
#if 0
    dlockdev_t	*user_copy;
#endif

    sp = &(pp->p_spools[k]);
#if 0
    user_copy = sp->sp_dldevlist;
#endif

#if 0
    size = sp->sp_dldevs * sizeof(dlockdev_t);
    sp->sp_dldevlist = (dlockdev_t *)kmem_alloc(size, KM_SLEEP);

    if (copyin(user_copy, sp->sp_dldevlist, size)) {
      goto dlock_backout;
    }
#endif

#if 0
    for (i=0; i < sp->sp_dldevs; i++) {
      nl = &(sp->sp_dldevlist[i]);
      nl->dl_dev = to_kdev_t(nl->dl_adev);
    }
#endif
  }

  for (j=0; j < pp->p_num_spools; j++) {
    device_t	*user_copy;

    sp = &(pp->p_spools[j]);

    user_copy = sp->sp_devlist;

    size = sp->sp_devs * sizeof(device_t);
    sp->sp_devlist = (device_t *)kmem_alloc(size, KM_SLEEP);

    if (copyin(user_copy, sp->sp_devlist, size)) {
      goto disk_backout;
    }
    
#if 0
    for (i=0; i < sp->sp_devs; i++) {
      nd = &(sp->sp_devlist[i]);
      nd->d_dev = to_kdev_t(nd->d_adev);
    }
#endif
  }

  return (0);

   
  /* back out the alloc's if there was an error */

 disk_backout:

    for (; j >= 0; j--) {
      sp = &(pp->p_spools[j]);
      kmem_free(sp->sp_devlist, sp->sp_devs * sizeof(device_t));
      sp->sp_devlist = NULL;
    }


 dlock_backout:

#if 0
    for (; k >= 0; k--) {
      sp = &(pp->p_spools[k]);
      kmem_free(sp->sp_dldevlist, sp->sp_dldevs * sizeof(dlockdev_t));
      sp->sp_dldevlist = NULL;
    }

    kmem_free(pp->p_spools, pp->p_num_spools * sizeof(subpool_t));
    pp->p_spools = NULL;
#endif

    return KRET(EFAULT);
}



/******************************************************************************
 ******************************************************************************
 **
 ** int add_pool()
 **
 ** Description:
 **	This routine reads the pool label information from user space and
 **	assigns a pool minor number to a newly created pool.  The arguement is
 **	a pointer to an pool_add_t structure which has been copied to kernel
 **	space.  All of the pointers in this structure reference addresses in
 **	user space.  If the pool label is already loaded this routine returns
 **	the minor number already assigned to the pool. It is returned in the
 **	pa_pool_dev field of the pool_add_t structure which is copied back out
 **	to user space by poolioctl upon return from this function.  If the
 **	label is not already loaded, pool_add finds an empty pool_t structure
 **	in the pools array and allocates it the new pool.  It then reads in the
 **	pool information and returns the minor number assigned to this new
 **	pool.  See the pool_t definition in pool_common.h for a description of
 **	the locking protocol for the pool structure.
 **
 ******************************************************************************
 *****************************************************************************/

int add_pool(pool_add_t *add)
{
  pool_t     		*new_pp = NULL;
  int         		 i;
  int         		 ret;
  int			 pl;


  IFDEBUG(POOL_DEBUG1,
	  printk("Entered pool_add()\n"););

  
  /* see if this pool has already been added */

  for (i=1; i < POOL_MAX_DEVICES; i++) { 

    POOL_SLOCK(pl, &pools[i].p_mutex, pldisk);

    if (pools[i].p_flags & PF_FREE) {
      POOL_SUNLOCK(&pools[i].p_mutex, pl);
      continue;
    }

    if (add->pa_pool_id == pools[i].p_pool_id) {
      if (pools[i].p_flags & PF_VALID) {
	POOL_SUNLOCK(&pools[i].p_mutex, pl);
	add->pa_pool_dev = makedevice(POOL_MAJOR,i);
	return 0;
      }
      else {
	POOL_SUNLOCK(&pools[i].p_mutex, pl);
	add->pa_pool_dev = makedevice(POOL_MAJOR,i);
	return KRET(EBUSY);
      }
    }
    else {
      POOL_SUNLOCK(&pools[i].p_mutex, pl);
    }
  }


  /* the pool is not already configured so assign a minor number to it */

  for (i=1; i < POOL_MAX_DEVICES; i++) {

    POOL_SLOCK(pl,&pools[i].p_mutex, pldisk);

    if (pools[i].p_flags & PF_FREE && pools[i].p_flags & PF_VALID) {

      pools[i].p_flags = PF_INIT; /* clear valid and free flags */
      ASSERT(!POOL_BUSY(&pools[i]));

      POOL_SUNLOCK(&pools[i].p_mutex, pl);
      break;
    } 
    else {
      POOL_SUNLOCK(&pools[i].p_mutex, pl);
    }
  }
  

  if (i == POOL_MAX_DEVICES) {
    return KRET(ENOMEM);
  }


  new_pp = &pools[i];

  /* clear out all the old stats */

#if 0
  memset(&new_pp->p_stats, 0, sizeof(pool_stat_t));
  memset(&new_pp->p_dstats, 0, sizeof(pool_dstat_t) * POOL_DSTATS_MAX);
  memset(&new_pp->p_dlstats, 0, sizeof(pool_dlstat_t));
#endif
#if 0
  add->pa_pool_dev = kdev_t_to_nr(makedevice(POOL_MAJOR, i));
#endif

  ret = gen_pool(add, new_pp);

  if (ret != 0) {
    POOL_SLOCK(pl,&new_pp->p_mutex, pldisk);
    new_pp->p_flags |= PF_VALID | PF_FREE;
    POOL_SUNLOCK(&new_pp->p_mutex, pl);
    return ret;
  }

  ret = open_devices(new_pp);

  if (ret != 0) {
    POOL_SLOCK(pl, &new_pp->p_mutex, pldisk);
    new_pp->p_flags |= PF_VALID | PF_FREE;
    POOL_SUNLOCK(&new_pp->p_mutex, pl);
    return ret;
  }

  IFDEBUG(POOL_DEBUG_ADD,
	  print_pool(new_pp););

  POOL_SLOCK(pl,&new_pp->p_mutex, pldisk);
  new_pp->p_flags |= PF_VALID | PF_LOWEROPEN;
  POOL_SUNLOCK(&new_pp->p_mutex, pl);

  return (0);
}


/******************************************************************************
*******************************************************************************
** NAME
**	free_pool()
**
** DESCRIPTION:  free memory associated with the pool_t
**
** RETURN VALUE
**
*******************************************************************************
******************************************************************************/

int free_pool(pool_t *pool)
{ 
  subpool_t           	*spoolp;
  int                 	j;


  IFDEBUG(POOL_DEBUG2,
	  printk("Entered free_pool\n"););

  for (j=0; j < pool->p_num_spools; j++) {
    spoolp = &(pool->p_spools[j]);
#if 0
    kmem_free(spoolp->sp_dldevlist, spoolp->sp_dldevs * sizeof(dlockdev_t));
    kmem_free(spoolp->sp_devlist, spoolp->sp_devs * sizeof(device_t));
#endif
  }

#if 0
  kmem_free(pool->p_spools, pool->p_num_spools * sizeof(subpool_t));
#endif
  pool->p_spools = NULL;

  return(0);
}


/******************************************************************************
*******************************************************************************
**
** int remove_pool()
**
** Description:
**	This routine checks that a pool is not in use and then removes it from
**	the list of managed pools.
**
*******************************************************************************
******************************************************************************/

int remove_pool(uint32_t minor)
{
  int 		pl;
  int		rc;
  pool_t	*pp;

  IFDEBUG(POOL_DEBUG1,
	  printk("Entered pool_remove\n"););

  pp = &pools[minor];

  POOL_SLOCK(pl,&pp->p_mutex, pldisk);
  if (pp->p_flags & PF_FREE) {

    POOL_SUNLOCK(&pp->p_mutex, pl);
    return KRET(EINVAL);
  }
  
  if (!(pp->p_flags & PF_VALID) || POOL_BUSY(pp)) {

    POOL_SUNLOCK(&pp->p_mutex, pl);
    return KRET(EBUSY);
  }
#if 0
  fsync_dev(makedevice(POOL_MAJOR,minor));
#endif

  if (pp->p_flags & PF_LOWEROPEN) {
    close_devices(pp);
  }

  rc = free_pool(pp);

  pp->p_flags = PF_VALID | PF_FREE;
  POOL_SUNLOCK(&pp->p_mutex, pl);
  IFDEBUG(POOL_DEBUG_REMOVE,
	  printk("pool (%d,%d) removed\n", POOL_MAJOR, minor););
  return (rc);
}


/******************************************************************************
*******************************************************************************
**
** int list_pool()
**
** Description:
**	This routine runs through the list of in-use pools and returns
**	information to the caller.  It provides a mapping between pool devices
**	and label devices and provides a versioning system by returning the
**	unix time at which the pool was last read in.  It also provides
**	information about whether the device is in use or not.
**
*******************************************************************************
******************************************************************************/

int list_pool(pool_list_t *list)
{
  int			 i,j,k;
  int			 pl;
  pool_status_t		*stats;
  pool_t		*pp;

  IFDEBUG(POOL_DEBUG1,
	  printk("Entered pool_list\n"););

  if (list->pl_count > POOL_MAX_DEVICES)
    list->pl_count = POOL_MAX_DEVICES;

  stats = (pool_status_t *) 
    kmem_alloc(list->pl_count * sizeof(pool_status_t), KM_SLEEP);
  memset(stats, 0, list->pl_count * sizeof(pool_status_t));

  for (i=1,j=0; i<POOL_MAX_DEVICES; i++) {

    pp = &pools[i];

    POOL_SLOCK(pl,&pp->p_mutex, pldisk);

    if (pp->p_flags & PF_FREE || !(pp->p_flags & PF_VALID)) {
      POOL_SUNLOCK(&pp->p_mutex, pl);
      continue;
    }

    stats[j].ps_pool_major = (uint32)POOL_MAJOR;
    stats[j].ps_pool_minor = (uint32)i;
    stats[j].ps_pool_id = pp->p_pool_id;
    stats[j].ps_dlocks = pp->p_dlocks;
    stats[j].ps_capacity = pp->p_capacity;
    for(k=0; k < POOL_NAME_SIZE; k++)
      stats[j].ps_pool_name[k] = pp->p_name[k]; 
    if (POOL_BUSY(pp))
      stats[j].ps_in_use = 1;
    else 
      stats[j].ps_in_use = 0;
    POOL_SUNLOCK(&pp->p_mutex, pl);

    IFDEBUG(POOL_DEBUG_LIST,
	    printk("  pool device: (%d,%d)\n", stats[j].ps_pool_major,
		   stats[j].ps_pool_minor);
	    printk("  pool id: 0x%qx\n", stats[j].ps_pool_id);
	    printk("  device is %sin use\n", stats[j].ps_in_use ? "" : "not ");
	    );

    if (++j >= list->pl_count)
      break;
  }

  list->pl_count = j;

  if (!(j > 0)) {
    kmem_free(stats, list->pl_count * sizeof(pool_status_t));
    return 0;
  }

  if (copyout(stats, list->pl_list, (int)(sizeof(pool_status_t) * j))) {
    kmem_free(stats, list->pl_count * sizeof(pool_status_t));
    return KRET(EFAULT);
  }

  kmem_free(stats, list->pl_count * sizeof(pool_status_t));
  return 0;
}


/******************************************************************************
*******************************************************************************
** NAME
**	list_spool
**
** DESCRIPTION
**	Generate a list of subpools for a given pool and copy their sizes out
**	to user space
**
** RETURN VALUE
**	Standard unix errno's
**
*******************************************************************************
******************************************************************************/

int list_spool(pool_t *pp, pool_splist_t *param)
{
  pool_spstatus_t	*list;
  subpool_t		*spool;
  int	i,j;
  int	max;
  device_t 		*devp;
  
  IFDEBUG(POOL_DEBUG1,
	  printk("Entering list_spool\n"););

  max = pp->p_num_spools > param->spl_count ? param->spl_count :
    pp->p_num_spools;

  list = kmem_alloc(max * sizeof(pool_spstatus_t), KM_SLEEP);
  memset(list, 0, max * sizeof(pool_spstatus_t));
  
  for(i=0; i < max; i++) {
    spool = &pp->p_spools[i];
      
    list[i].sps_boffset = spool->sp_start;
    list[i].sps_bcount = spool->sp_len;
    list[i].sps_stripe = spool->sp_stripe;
    list[i].sps_dlstart = spool->sp_dlstart;
#if 0
    list[i].sps_dlocks = spool->sp_dlocks;
#endif
    list[i].sps_numdevs = spool->sp_devs;
    list[i].sps_dldevs = spool->sp_dldevs;

    /* hack so we can get dev list for pinfo */
    if (spool->sp_devs <= 64){ 
      for(j=0; j< spool->sp_devs; j++){
	devp = &(spool->sp_devlist[j]);
	list[i].sps_minors[j] = MINOR(devp->d_dev);
      }
    }    
  }

  if (copyout(list, param->spl_list, (int)(sizeof(pool_spstatus_t) * max))) {
    kmem_free(list, sizeof(pool_spstatus_t) * max);
    return KRET(EFAULT);
  }

  param->spl_count = max;

  kmem_free(list, sizeof(pool_spstatus_t) * max);
  return (0);
}


/******************************************************************************
*******************************************************************************
** NAME
**	pool_blkmap
**
** DESCRIPTION  psector in basic blocks, pbcount in bytes
**
** RETURN VALUE 
**
*******************************************************************************
******************************************************************************/

int pool_blkmap(pool_t *pp, uint64 psector, uint32 pbcount, 
		uint64 *dev_sector, uint32 *dev_bcount, device_t **out_dev)
{
  subpool_t           	*sp = NULL;
  device_t            	*devp	= NULL;
  int			i,j;
  uint32		current_spool = 0;
  uint32		current_dev;
  uint64		blk;
  uint32		devnum = 0;    /* # devices in selected spool */
  
  IFDEBUG(POOL_DEBUG_MAP,
	  printk("Entered pool_blkmap: psec=%ld, pbcnt=%u\n",
		 psector, pbcount););

  /* select subpool */

  for (j=current_spool; j<pp->p_num_spools; j++) {
    sp = &(pp->p_spools[j]);

    if ((psector >= sp->sp_start) &&
	(psector < sp->sp_start+sp->sp_len)) {
      break;
    }
  }

  if (j == pp->p_num_spools) {
    printk("block not in any subpools at line %d!\n", __LINE__);
    return KRET(EIO);
  }
  
  current_spool = j;
  devnum = sp->sp_devs;

  IFDEBUG(POOL_DEBUG_MAP,
	  printk("  spl=%u, ", current_spool););


  /* Determine device within selected sub-pool */

  if (sp->sp_stripe) {

    IFDEBUG(POOL_DEBUG_MAP,
	    printk("str=%u, ", sp->sp_stripe););

    if ((sp->sp_stripe<<BBSHIFT) % (1<<BBSHIFT) != 0) {
      printk("stripe width must be a mutliple of the basic block");
      return KRET(EIO);
    }
    current_dev = (uint32)(((psector - sp->sp_start) 
			    / sp->sp_stripe) % devnum);
  } 
  else {
    for (i=0; i<devnum; i++) {
      devp = &(sp->sp_devlist[i]);
      
      if (((psector - sp->sp_start) >= devp->d_start) &&
	  ((psector - sp->sp_start) < (devp->d_start + devp->d_nblks)))
	break;
    }

    if (i == devnum) {
      printk("\nblock requested past end of subpool at line %d!\n", __LINE__);
      IFDEBUG(POOL_DEBUG_MAP,
	      printk("psector: %qd, sp_start: %qd, d_start: "
		     "%qd, d_nblks: %qd\n", psector, 
		     sp->sp_start, devp->d_start, devp->d_nblks););
      return KRET(EIO);
    }

    current_dev = i;
  }
  IFDEBUG(POOL_DEBUG_MAP,
	  printk("dev=%u, ", current_dev););


  devp = &(sp->sp_devlist[current_dev]);


  /* Determine the block number on the device */

  if (sp->sp_stripe) {
    blk = (((psector - sp->sp_start) / (sp->sp_stripe*sp->sp_devs)) * 
	   sp->sp_stripe) + ((psector - sp->sp_start) % sp->sp_stripe);
  } 
  else {
    blk = psector - sp->sp_start - devp->d_start;
  }
  IFDEBUG(POOL_DEBUG_MAP,
	  printk("blk=%d\n", blk););


  /* account for partition header */

  /* blk += (SIZEOF_POOL_LABEL >> 9); */

  *dev_sector = blk;
  *out_dev = devp;
  
  return (0);
}
