/******************************************************************************
*******************************************************************************
**
**    Copyright 1995-1999 Regents of the University of Minnesota
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      David C. Teigland  <teigland@lcse.umn.edu>
** 
**    Module name: pool_debug.h
**
**    Description:
**      Header for pool driver debugging code.
**
*******************************************************************************
******************************************************************************/

#ifndef __POOL_DEBUG_H__
#define __POOL_DEBUG_H__

#include "global.h"


/* Pool Debugging Flags */

#define POOL_DEBUG1             (0x0000000000000001)
#define POOL_DEBUG2             (0x0000000000000002)

#define POOL_DEBUG_OPEN	        (0x0000000000000004)

#define POOL_DEBUG_LOCK1        (0x0000000000000008)
#define POOL_DEBUG_LOCK2        (0x0000000000000010)
#define POOL_DEBUG_LOCK3        (0x0000000000000020)
#define POOL_DEBUG_LOCK_TIME	(0x0000000000000040)

#define POOL_DEBUG_ADD		(0x0000000000000100)
#define POOL_DEBUG_LIST		(0x0000000000000200)
#define POOL_DEBUG_REMOVE	(0x0000000000000400)

#define POOL_DEBUG_TIME         (0x0000000000000040)
#define POOL_DEBUG_MAP	        (0x0000000000000080)
#define POOL_DEBUG_SLOCK	(0x0000000000001000)
#define POOL_DEBUG_BUFT		(0x0000000000002000)


#ifdef WANT_DFLAG_T
const debugFlag_t POOL_DFLAGS[] = 
{
  {"pool1", 		POOL_DEBUG1},
  {"pool2", 		POOL_DEBUG2},
  {"pool", 		(POOL_DEBUG1 | POOL_DEBUG2)},
  
  {"pool_open",		POOL_DEBUG_OPEN},

  {"pool_lock1",	POOL_DEBUG_LOCK1},
  {"pool_lock2",	POOL_DEBUG_LOCK2},
  {"pool_lock3",	POOL_DEBUG_LOCK3},
  {"pool_lock_time",	POOL_DEBUG_LOCK_TIME},
  {"pool_lock",		(POOL_DEBUG_LOCK1 | POOL_DEBUG_LOCK2 | 
			 POOL_DEBUG_LOCK3)},

  {"pool_time", 	POOL_DEBUG_TIME},
  {"pool_map",	 	POOL_DEBUG_MAP},
  {"pool_add",          POOL_DEBUG_ADD},
  {"pool_list",		POOL_DEBUG_LIST},
  {"pool_remove",	POOL_DEBUG_REMOVE},
  {"pool_slock", 	POOL_DEBUG_SLOCK},
  {"pool_buft",		POOL_DEBUG_BUFT},

  {"all", 		0xFFFFFFFFFFFFFFFFll},
  {"none", 		0x0},

  {"",                  0x0}  /*  This must be the last flag.  */
};
#endif

#ifndef __POOL_DEBUG_C__
extern uint64 pool_dflags;
#endif

#ifdef DEBUG
#define IFDEBUG(flag, x1) {if ((pool_dflags & flag)) {x1}}
#else
#define IFDEBUG(flag, x1)
#endif

#endif /* __POOL_DEBUG_H__ */
