/******************************************************************************
*******************************************************************************
**
**    Copyright 1995-1998 Regents of the University of Minnesota
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      David C. Teigland  <teigland@lcse.umn.edu>
** 
**    Module name: pool_disk.h
**
**    Description:
**	Network storage pool device driver shared header file, on disk
**	structures.
**
*******************************************************************************
******************************************************************************/

#ifndef __POOL_DISK_H__
#define __POOL_DISK_H__

#ifdef __cplusplus
extern "C" {
#endif

  /* Preprocessor Defines */

#define POOL_NAME_SIZE		256

  /* Function Macros */

  /* Type Definitions */

  /* Device description - NOT devices in a subpool must have the same d_nblks */

  typedef struct disk_device_s
  {
    uint64	dd_start;	/* subpool address of 1st block */
    uint64	dd_nblks;	/* # of blocks */
    uint32	dd_offset;
  } disk_device_t;

  /* Subpool description */

  typedef struct disk_subpool_s
  {
    uint64	dsp_start;	/* lba of 1st block */
    uint32	dsp_len;	/* # of blocks in subpool */
    uint32	dsp_striping;	/* in terms of blocks */
    uint32	dsp_devices;	/* # of devices */
  } disk_subpool_t;

  /* Pool partion header, pool_assemble assumes this can fit in one basic 
   * block */

  typedef struct disk_partition_header_s
  {
    uint64	ph_magic;	/* partion header */       
    uint64	ph_pool_id;	/* pool identifier */
    uint32	ph_version;	/* structure versioning info. */
    char	ph_name[POOL_NAME_SIZE]; /* pool name */
    uint32	ph_spool_num;	/* sub-pool number */
    uint32	ph_spool_dev;	/* sub-pool device number */
    int32	ph_dl_id;	/* -1 if no locks on device */
  } disk_partition_header_t;

  /* Disk lock device descriptions */

  typedef struct disk_dlockdev_s
  {
    uint64	ddl_wwn;	/* WWN for fc devices, otherwise 0 */
    uint32	ddl_start;	/* address of 1st lock */
    uint32	ddl_number;	/* # of locks */
    uint32	ddl_netaddr;
  } disk_dlockdev_t;

  /* Storage pool labels, pool_assemble assumes this can fit in one basic 
   * block */

  typedef struct disk_pool_labels_s
  {
	  
    uint64	pl_magic;	/* label magic number */
    uint64	pl_pool_id;	/* pool identifier */
    uint32	pl_version;	/* pool version */
    uint32	pl_labelsize;	/* # of BB blocks for labels */
    char       	pl_devlabel[POOL_NAME_SIZE];
    
    uint32	pl_subpools;	/* # of subpools */
    uint32	pl_dlocks;	/* # of device locks */
    uint32	pl_dldevices;	/* # of device lock devices */
    uint32	pl_dlstriping;	/* are device locks striped? */
    
    uint32	pl_devices;	/* # of lower devices */
    uint64	pl_capacity;	/* # of blocks */
  } disk_pool_labels_t;

#ifdef __cplusplus
}
#endif

#endif /* __POOL_DISK_H__ */

