/******************************************************************************
*******************************************************************************
**
**    Copyright 1999 Sistina Software, Inc.  <kpreslan@sistina.com>
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Kenneth W. Preslan <kpreslan@sistina.com>
** 
**    Module name: do_dlock-0.9.4.c
**
*******************************************************************************
******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include "scsi_lib.h"



#define T (1)
#define F (0)





int main(int argc, char *argv[])
{
  int fd;
  unsigned char cdb[16], data[16384];
  unsigned int action, locknumber, clientid, inputversionnumber;
  int result, x, y, len;



  if (argc < 6)
  {
    fputs("do_dlock-0.9.4 DeviceName Action LockNumber ClientID InputVersionNumber\n", stderr);
    fputs("\noutput for all actions except 9:\n", stderr);
    fputs("  Result Activity Expired State Version Holders\n", stderr);
    fputs("  Holder0 Holder1 Holder2 Holder3...\n", stderr);
    fputs("\noutput for action 9:\n", stderr);
    fputs("  Result\n", stderr);
    fputs("  Lock0 Lock1 Lock2 Lock3 Lock4...\n",stderr);
    exit(EXIT_FAILURE);
  }



  if ((fd = scsi_open(argv[1])) < 0)
  {
    fprintf(stderr, "do_dlock-0.9.4:  can't open device %s:  %s\n", argv[1], strerror(errno));
    exit(EXIT_FAILURE);
  }

  action = atoi(argv[2]);
  locknumber = atoi(argv[3]);
  clientid = atoi(argv[4]);
  inputversionnumber = atoi(argv[5]);

  memset(data, 0, 16384);



  memset(cdb, 0, 16);

  cdb[0] = 0x83;

  cdb[1] = action & 0x0F;

  cdb[2] = (locknumber >> 24) & 0xFF;
  cdb[3] = (locknumber >> 16) & 0xFF;
  cdb[4] = (locknumber >> 8) & 0xFF;
  cdb[5] = (locknumber >> 0) & 0xFF;

  cdb[6] = (clientid >> 24) & 0xFF;
  cdb[7] = (clientid >> 16) & 0xFF;
  cdb[8] = (clientid >> 8) & 0xFF;
  cdb[9] = (clientid >> 0) & 0xFF;

  cdb[10] = 0x00;
  cdb[11] = 0x00;
  cdb[12] = 0x40;
  cdb[13] = 0x00;

  cdb[14] = inputversionnumber & 0xFF;

  cdb[15] = 0x00;

#ifdef DEBUG
  for (x = 0; x < 16; x++)
    fprintf(stderr, "%.2X ", cdb[x]);
  fprintf(stderr, "\n");
#endif  /*  DEBUG  */



  if ((result = scsi_cmd(fd, 16, cdb, 0, NULL, 16384, data, 0, 0)) < 0)
  {
    fputs("do_dlock-0.9.4:  bad dlock command\n", stderr);
    exit(EXIT_FAILURE);
  }

  

#ifdef DEBUG
  fprintf(stderr, "Dlock ok:  %d\n", result);
  for (x = 0; x < result; x++)
  {
    fprintf(stderr, "%.2X ", data[x]);
    if (x % 16 == 15)
      fprintf(stderr, "\n");
  }
  if (x % 16)
    fprintf(stderr, "\n");
#endif  /*  DEBUG  */

  if (action != 9)
  {
    printf("%u_result ", (data[4] >> 7) & 0x01);
    printf("%u_activity ", (data[4] >> 6) & 0x01);
    printf("%u_expired ", (data[4] >> 2) & 0x03);
    printf("%u_state ", (data[4] >> 0) & 0x03);
    printf("%u_version ",
	   (data[0] << 24) |
	   (data[1] << 16) |
	   (data[2] << 8) |
	   (data[3] << 0));
    printf("%u_holders ", data[5]);
    
    printf("\n");
    
    len = ((data[6] << 8) | (data[7] << 0)) / 4;
    
    for (x = 0; x < len; x ++)
      printf("%u ",
	     (data[4 * x + 8] << 24) |
	     (data[4 * x + 9] << 16) |
	     (data[4 * x + 10] << 8) |
	     (data[4 * x + 11] << 0));
    
    printf("\n");
  }
  else
  {
    printf("%u_result\n", (data[0] >> 7) & 0x01);
    
    len = (data[2] << 8) | (data[3] << 0);
    
    for (x = 0; x < len; x++)
      for (y = 0; y < 8; y++)
	printf("%u ", (data[x + 4] & (1 << y)) ? 1 : 0);
    
    printf("\n");
  }



  scsi_close(fd);

  exit(EXIT_SUCCESS);
}

