/******************************************************************************
*******************************************************************************
**
**    Copyright 1999 Sistina Software, Inc.  <kpreslan@sistina.com>
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Kenneth W. Preslan <kpreslan@sistina.com>
** 
**    Module name: do_modepage.c
**
*******************************************************************************
******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include "scsi_lib.h"



#define T (1)
#define F (0)

#define BIGSIZE (16384)





int main(int argc, char *argv[])
{
  int fd;
  unsigned char cdb[16], data[BIGSIZE];
  unsigned int pc, timeout = 0, save = 0, do_write = F;
  int result, off;



  if (argc < 3)
  {
    fputs("mode_page DeviceName PC <Save> <Timeout>\n", stderr);
    fputs("\noutput:\n", stderr);
    fputs("  ClientsPerLock NumberOfLocks LockTimeoutInterval\n", stderr);
    exit(EXIT_FAILURE);
  }



  if ((fd = scsi_open(argv[1])) < 0)
  {
    fprintf(stderr, "do_dlock:  can't open device %s:  %s\n", argv[1], strerror(errno));
    exit(EXIT_FAILURE);
  }

  pc = atoi(argv[2]);
  if (argc >= 5)
  {
    do_write = T;
    save = atoi(argv[3]);
    timeout = atoi(argv[4]);
  }

  memset(data, 0, BIGSIZE);



  memset(cdb, 0, 16);

  cdb[0] = 0x1A;
  cdb[1] = 0x00;
  cdb[2] = (pc << 6) | 0x29;
  cdb[3] = 0x00;
  cdb[4] = 0xFF;
  cdb[5] = 0x00;

#ifdef DEBUG
  for (x = 0; x < 16; x++)
    fprintf(stderr, "%.2X ", cdb[x]);
  fprintf(stderr, "\n");
#endif  /*  DEBUG  */

  

  if ((result = scsi_cmd(fd, 6, cdb, 0, NULL, 255, data, 0, 0)) < 0)
    fputs("mode_page:  bad mode_sense command\n", stderr);
  else
  {
#ifdef DEBUG
    fprintf(stderr, "mode_sense ok:  %d\n", result);
    for (x = 0; x < result; x++)
    {
      fprintf(stderr, "%.2X ", data[x]);
      if (x % 16 == 15)
	fprintf(stderr, "\n");
    }
    if (x % 16)
      fprintf(stderr, "\n");
#endif  /*  DEBUG  */


    off = 4 + data[3];

    printf("%uclients ", data[off + 3]);
    printf("%ulocks ",
	   (data[off + 4] << 24) |
	   (data[off + 5] << 16) |
	   (data[off + 6] << 8) |
	   (data[off + 7] << 0));
    printf("%utimeout\n",
	   (data[off + 8] << 24) |
	   (data[off + 9] << 16) |
	   (data[off + 10] << 8) |
	   (data[off + 11] << 0));


    if (do_write)
    {
      memset(cdb, 0, 16);
      
      cdb[0] = 0x15;
      cdb[1] = 0x10 | ((save) ? 0x01 : 0x00);
      cdb[2] = 0x00;
      cdb[3] = 0x00;
      cdb[4] = 16;
      cdb[5] = 0x00;

#ifdef DEBUG
      for (x = 0; x < 16; x++)
	fprintf(stderr, "%.2X ", cdb[x]);
      fprintf(stderr, "\n");
#endif  /*  DEBUG  */


      memset(data, 0, BIGSIZE);

      data[4] = 0x29;
      data[5] = 0x0A;

      data[12] = (timeout >> 24) & 0xFF;
      data[13] = (timeout >> 16) & 0xFF;
      data[14] = (timeout >> 8) & 0xFF;
      data[15] = (timeout >> 0) & 0xFF;
     

#ifdef DEBUG
      fprintf(stderr, "mode_select data\n");
      for (x = 0; x < 16; x++)
	fprintf(stderr, "%.2X ", data[x]);
      fprintf(stderr, "\n");
#endif  /*  DEBUG  */


      if ((result = scsi_cmd(fd, 6, cdb, 16, data, 0, NULL, 0, 0)) < 0)
	fputs("bad mode_select command\n", stderr);
    }
  }



  scsi_close(fd);

  exit(EXIT_SUCCESS);
}
