#ifndef __PARTITION_H__
#define __PARTITION_H__

#include "global.h"
#include "mode_page.h"
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include "dlock_CDB.h"
#include <errno.h>

struct server_dlock; typedef struct server_dlock server_dlock;
struct partition_locks; typedef struct partition_locks partition_locks; 

partition_locks *new_partition_locks(int numlocks);
server_dlock    *new_server_dlock();


/*********************************************************
 *********************************************************
 **get_cid_list()
 **
 **Description: return the ith cid_entry in the server
 **dlock. 
 **Returns: cid_entry pointer. 
 *********************************************************
 *********************************************************/
cid_entry *get_cid_list(int i, server_dlock *sdl);

/*********************************************************
 *********************************************************
 **get_cid_index()
 **
 **Description: returns the index of the input cid value
 **within the input server dlock. 
 *********************************************************
 *********************************************************/
int  get_cid_index(uint32 lookup, server_dlock *sdl);

/*********************************************************
 *********************************************************
 **sdl_reset()
 **
 **Description: updates the lock's expire time. 
 *********************************************************
 *********************************************************/
void sdl_reset       (server_dlock *sdl);

/*********************************************************
 *********************************************************
 **sdl_clear_cid()
 **
 **Description: removes all client id's from a dlock.
 *********************************************************
 *********************************************************/
void sdl_clear_cid   (server_dlock *sdl);

/*********************************************************
 *********************************************************
 **add_dlock_cid()
 **
 **Description: adds a client id to a dlock rdf.
 **Returns:1 always
 *********************************************************
 *********************************************************/
int  add_dlock_cid   (server_dlock *sdl, uint32 add);

/*********************************************************
 *********************************************************
 **rem_dlock_cid()
 **
 **Description: removes the input client id from a dlock.
 **
 **Input: uint32 of client id value.
 **Returns: -1 on error, 1 on success.
 *********************************************************
 *********************************************************/
int  rem_dlock_cid   (server_dlock *sdl, uint32 rem);

/*********************************************************
 *********************************************************
 **cid_in_dlock()
 **
 **Description: returns true if the cid is in the lock, 
 ** false if not.
 **
 **Input: uint32 of cid value.
 *********************************************************
 *********************************************************/
int  cid_in_dlock    (server_dlock *sdl, uint32 lookup);

/*********************************************************
 *********************************************************
 **delete_partition_locks()
 **
 **Description: like a deconstructor function.
 *********************************************************
 *********************************************************/
void delete_partition_locks(partition_locks *pal);

/*********************************************************
 *********************************************************
 **delete_server_dlock()
 **
 **Description: deconstructor function.
 *********************************************************
 *********************************************************/
void delete_server_dlock(server_dlock *sdl);


/***********************************************************************
 ***********************************************************************
 *struct server_dlock  DATA TYPE   
 *
 *each server dlock keeps track of all state information for a single dlock. 
 *client ids are stored in a linked list. Since the average number of
 *clients holding each lock is much less than one, this implementation offers
 *no performance penalty.
 *
 ***********************************************************************
 ***********************************************************************/
struct server_dlock{
  uint32                 lock_number;
  uint32                 version_number;
  uint8                  state;
  uint8                  act;
  uint8                  num_holders;
  uint8                  expired;
  uint8                  exclusive_pending;
  double                 expire_time;
  cid_entry              *head;
};


/**********************************************************
 **********************************************************
 **********************************************************
 ***DATA TYPE 
 *** struct partition_locks
 ***
 *** data structure used to hold all relevant information about
 *** a server's dlocks. A partition_locks data
 *** structure can hold any number of locks.
 ***
 **********************************************************
 **********************************************************/
struct partition_locks{
  struct server_dlock   **sdl;
  uint32                dlocks_per_partition;
  uint32                lock_timeout_interval;
  uint8                 max_lock_holds;

};





#endif /* __PARTITION_C__ */
