/******************************************************************************
*******************************************************************************
**
**    Copyright 1995-1999 Regents of the University of Minnesota
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Andrew Barry <barry@lcse.umn.edu>
** 
**    Module name: global.h
**
**    Description: global declarations for gfs_ipd
**
*******************************************************************************
******************************************************************************/


#ifndef __GLOBAL_H__
#define __GLOBAL_H__

/* #include <sgidefs.h> */
#include <sys/types.h>
#include <pthread.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/wait.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <assert.h>
/* remove this */
/*#include "memory.h"*/

typedef   u_int             uint32;
typedef   u_short           uint16;
typedef   u_char            uint8;

typedef   uint32            u32;
typedef   uint16            u16;
typedef   uint8             u8;


#define FIRSTBIT                (1<<0) /*  0x01 */
#define SECONDBIT               (1<<1) /*  0x02 */
#define THIRDBIT                (1<<2) /*  0x04 */
#define FOURTHBIT               (1<<3) /*  0x08 */
#define FIFTHBIT                (1<<4) /*  0x10 */
#define SIXTHBIT                (1<<5) /*  0x20 */
#define SEVENTHBIT              (1<<6) /*  0x40 */
#define EIGHTHBIT               (1<<7) /*  0x80 */

#define TIME_QUANTUM            (10)
#define EXPIRE_TIME             (5000)

#define MAX_DRV_USERS           (8)
#define MAX_USERS               (8)
#define MAX_LOCK_HOLDS          (8)
#define DLOCKS_PER_PARTITION    (1024)
#define PALS_PER_SERVER         (8)

#define ACTION_NOP              (0)
#define ACTION_LOCK_S           (1)
#define ACTION_LOCK_E           (2)
#define ACTION_FORCE_LOCK_E     (3)
#define ACTION_TOUCH            (4)
#define ACTION_UNLOCK           (5)
#define ACTION_UNLOCK_I         (6)
#define ACTION_ACTIVITY_ON      (7)
#define ACTION_ACTIVITY_OFF     (8)

#define STATE_UNLOCKED          (0)
#define STATE_LOCKED_S          (1)
#define STATE_LOCKED_E          (2)

#define EXPIRED_NOT             (0)
#define EXPIRED_S               (1)
#define EXPIRED_E               (2)

#define EXCLUSIVE_PENDING       (1)
#define EXCLUSIVE_NOT_PENDING   (0)

#define ACTIVITY_OFF            (0)
#define ACTIVITY_ON             (1)

#define FLAGS_EX                (1<<0)
#define FLAGS_SH                (1<<1)
#define FLAGS_ALL               (1<<2)


/* using cc (not gcc) switch the commented and uncommented defines*/
#ifdef VERBOSE
#define dbprintf(fmt, args...) printf(fmt, ## args)
/*#define dbprintf printf*/
#else 
#define dbprintf(fmt, args...)
/*#define dbprintf(fmt, ...)*/
#endif

/* Values for the code field */
#define CodeLess 0x0
#define CODELESS Codeless
#define CDBdlock 0x01
#define CDBDLOCK CDBdlock
#define RDFdlock 0x11
#define RDFDLOCK RDFdlock
#define RDFexprd 0x13
#define RDFEXPRD RDFexprd
#define CDBmoder 0x04
#define CDBMODER CDBmoder
#define CDBmodew 0x08
#define CDBMODEW CDBmodew
#define RDFmode  0x14
#define RDFMODE  RDFmode
#define CallBack 0x20
#define CALLBACK CallBack


#define LCKOUT_HELD         (0x00000001)
#define LCKOUT_SHARED       (0x00000002)
#define LCKOUT_CACHEABLE    (0x00000004)
#define LCKOUT_EXPIRED_S    (0x00000008)
#define LCKOUT_EXPIRED_E    (0x00000010)
#define LCKOUT_NEED_S       (0x00000020)
#define LCKOUT_NEED_E       (0x00000040)


#define ULCK_MODIFIED       (0x00000001)
#define ULCK_RESETEXPIRED   (0x00000002)


#define CB_NEED_S           LCKOUT_NEED_S
#define CB_NEED_E           LCKOUT_NEED_E
#define CB_EXPIRED_S        LCKOUT_EXPIRED_S
#define CB_EXPIRED_E        LCKOUT_EXPIRED_E
#define CB_FREED_S          (0x00000080)
#define CB_FREED_E          (0x00000100)


struct dlock_CDB {
  u8  code;
  u8  action;
  u8  input_version_number;
  u32 lock_number;
  u32 cid;
};
typedef struct dlock_CDB dlock_CDB;
typedef dlock_CDB dlock_cdb;


struct mode_page {
  u8  code;
  u8  max_clients;
  u32 num_locks;
  u32 lock_time_ms;
};
typedef struct mode_page mode_page;
typedef mode_page mode_CDB;
typedef mode_page mode_cdb;
typedef mode_page mode_page_RDF;/*excessive typedefs support*/ 
typedef mode_page mode_page_rdf;/*legacy code*/


struct dlock_RDF {
  u8   code;
  u8   result;
  u8   act;
  u8   expired;
  u8   state;
  u8   num_holders;
  u32  version_number;
};
typedef struct dlock_RDF dlock_RDF;
typedef dlock_RDF dlock_rdf;


struct rexpired_RDF {
  u8   code;
  u16  return_length;
  u8  *exlock_block;
};
typedef struct rexpired_RDF rexpired_RDF;
typedef rexpired_RDF rexpired_rdf;


struct drop_RDF {
  u8    code;
  u32   lock_number;
  u32   flags;
};
typedef struct drop_RDF drop_RDF;
typedef drop_RDF drop_rdf;

typedef struct ct {
	int sock;
	pthread_mutex_t sender;
} send_req_t;

int send_request(send_req_t *sr, char *buf, int sze);

#endif /*__GLOBAL_H__*/
