/******************************************************************************
*
*    Copyright 1999 Regents of the University of Minnesota
*
*    This is free software released under the GNU General Public License.
*    There is no warranty for this software.  See the file COPYING for
*    details.
*
*    See the file CONTRIBUTORS for a list of contributors.
*
*    This file is maintained by:
*      Mike C Tilstra <tilstra@lcse.umn.edu>
* 
*/


#include <pthread.h>
#include <assert.h>

/*#include "dlip_data_structures.h"*/
#include "global.h"
#include "partition_opps.h"


#define DLTCP_PORT 14242



partition_locks *nddpal;

int server_runs = 1;
/******************************************************************************
 * clientTalk()
 *
 * This is where all action is handled.
 * We wait for some data, where we pull off the first byte and check the
 * command code.  This tells us how many more bytes to pull off.
 * Then we can step out to handle the different data types, and return what
 * data is needed in return.
 *
 * This first implementation is rather a quick hack.
 */
void *clientTalk(void *args)
{
  int *t = (int*)args;
  int sk = *t;
  int n, sze;
  char ibuf[24]; /* bigger than we need */
  char obuf[24]; /* bigger than we need */
  send_req_t sr_data;
  mode_page mcdb, mrdf;
  dlock_cdb dcdb;
  dlock_rdf drdf;
  
  /* setup data space for send_request */
  sr_data.sock   = sk;
  /*sr_data.sender = PTHREAD_MUTEX_INITIALIZER;*/
  pthread_mutex_init(&(sr_data.sender), NULL);
  
  while(server_runs) {
    memset(ibuf, 0, 24);
    memset(obuf, 0, 24);
    
    /*check what kind of structure we're getting*/
    n = recv(sk, ibuf, 1, MSG_PEEK);
    if(n <0) perror("first recv gave an error");
    
    if(n==0){
      close(sk);
      pthread_exit(0);
    }
    if(ibuf[0] == CDBdlock)
      sze = sizeof(dlock_cdb);
    else if(ibuf[0] == CDBmoder || ibuf[0] == CDBmodew)
      sze = sizeof(mode_page);
    else
      sze = 0; /* need to handle this error */
    
    
    /* Now pull that in */
    n = recv(sk, ibuf, sze, 0);
    if(n < 0) perror("second recv gave an error");
    if(n==0){
      close(sk);
      pthread_exit(0);
    }
    /* handle data */
    sze = 0;
    
    /*dbprintf("working");*/
    
    if(ibuf[0] == CDBmoder || ibuf[0] == CDBmodew) {
      /* Do a mode page */
      mode_page *cdb = (mode_page *)ibuf;
      mode_page *rdf = (mode_page *)obuf;
      
      dbprintf("mode_page\n");			
      mcdb.num_locks=ntohl(cdb->num_locks);
      mcdb.lock_time_ms=ntohl(cdb->lock_time_ms);
      mcdb.code=cdb->code;
      mcdb.max_clients=cdb->max_clients;
      mode_actions(nddpal, &mcdb, &mrdf);
      rdf->num_locks=htonl(mrdf.num_locks);
      rdf->lock_time_ms=htonl(mrdf.lock_time_ms);
      rdf->max_clients=mrdf.max_clients;
      
      rdf->code = RDFmode;
      sze = sizeof(mode_page);
    }
    
    if(*ibuf == CDBdlock) {
      /* Do dlock request */
      dlock_cdb *cdb = (dlock_cdb*)ibuf;
      dlock_rdf *rdf = (dlock_rdf*)obuf;
      
      dcdb.lock_number=ntohl(cdb->lock_number);
      dcdb.cid=ntohl(cdb->cid);
      dcdb.action=cdb->action;
      dcdb.input_version_number=cdb->input_version_number;
      print_dlock_cdb(&dcdb);
      part_actions(nddpal, &dcdb, &drdf, &sr_data);
      print_dlock_rdf(&drdf);
      rdf->version_number=htonl(drdf.version_number);
      rdf->result=drdf.result;
      rdf->act=drdf.act;
      rdf->expired=drdf.expired;
      rdf->state=drdf.state;
      rdf->num_holders=drdf.num_holders;


      rdf->code = RDFdlock;
      sze = sizeof(dlock_rdf);
    }
    
	 pthread_mutex_lock( &sr_data.sender);
    n = send(sk, obuf, sze, 0);
	 pthread_mutex_unlock( &sr_data.sender);
    if(n <0) perror("send gave an error");
  }
  close(sk);
  pthread_exit(0);
  return(0);
}

/******************************************************************************
 * send_request()
 * fires off a packet of info to a given client.
 */

int send_request(send_req_t *sr, char *buf, int sze)
{
	int err;
	/*dbprintf("callback\t");*/
	assert( sr != NULL );
	
	pthread_mutex_lock( &sr->sender);
	err = send(sr->sock, buf, sze, 0);
	pthread_mutex_unlock( &sr->sender);
	return err;
}


/******************************************************************************
 * main()
 *
 */
int main(int argc, char **argv) {
  int skin, sknew, len, timel;
  struct sockaddr_in  myaddr, theiraddr;
  pthread_t tmpTid;
  
  
  if(argc<2){
    printf("gfs_ipd <numlocks> <timeout_interval>\n");
    printf("gfs_ipd <numlocks>\n");
    exit(1);
  }
  if(argc<3){  
    timel=60;} /* default lock timeout.  */
  else{
    timel=atoi(argv[2]);
  }
  
  /*initialize the locks.*/
  nddpal=new_partition_locks(atoi(argv[1]));
  nddpal->lock_timeout_interval=timel*1000;
  
  
  if( (skin = socket(AF_INET, SOCK_STREAM, 0)) <0 ) {
    perror("main: Socket biffed.\n");
    exit(1);
  }
  
  /* Set myself up to listen to the world. */
  myaddr.sin_family=AF_INET;
  myaddr.sin_addr.s_addr=INADDR_ANY;
  myaddr.sin_port=htons(DLTCP_PORT);
  if( bind(skin, (struct sockaddr *)&myaddr, sizeof(struct sockaddr_in)) <0) {
    perror("main: Bind Biffed.\n");
    exit(1);
	}
  
  if( listen(skin, 5) <0 )
    perror("main: Listen Biffed.\n");
  
  for(;;) {
    len = sizeof(struct sockaddr_in);
    if( (sknew = accept(skin, (struct sockaddr *)&theiraddr, &len)) <0)
      perror("main: Accept Biffed.\n");
    
    if( pthread_create( &tmpTid, NULL, clientTalk, (void *)&sknew) <0)
      perror("main: pthread_create");
    if( pthread_detach( tmpTid ) <0)
      perror("main: pthread_detach");
    
  }
}


