/******************************************************************************
*******************************************************************************
**
**    Copyright 1995-1999 Regents of the University of Minnesota
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Andrew Barry <barry@lcse.umn.edu>
** 
**    Module name: print.c
**
**    Description: bug checking functions for verbose mode of the server. 
**
*******************************************************************************
******************************************************************************/


#ifndef __PRINT_C__
#define __PRINT_C__


#include "print.h"



/***************************************
 *these functions print data structures from the 
 *gfs_ip server. All for the sake of bug checking.
 ***************************************/


void print_server_dlock(server_dlock *sdl){

  dbprintf("\n:sdl:");
  dbprintf("lock:%d  ",sdl->lock_number);
  dbprintf("version:%d  ",sdl->version_number);
  dbprintf("state:%d  act:%d  ",sdl->state, sdl->act);
  dbprintf("expired:%d  ",sdl->expired);
  dbprintf("excl:%d  ",sdl->exclusive_pending);
  dbprintf("exptime:%f  ",sdl->expire_time);
  dbprintf("holders:%d\n",sdl->num_holders);
}

void print_partition_locks(partition_locks *pal){
  int i,j;

  j=0;  
  dbprintf("\n::::::::partition_locks::::::::\n");
  for(i=0; i<DLOCKS_PER_PARTITION; i++){
    if(pal->sdl[i]->num_holders!=0){
      dbprintf("%d:%d\t",i,pal->sdl[i]->num_holders);
      if(++j>6){
	dbprintf("\n");
	j=0;
      }
    }
  }
  dbprintf("\n");
}



void print_dlock_cdb(dlock_cdb *cdb){
  dbprintf("action:%d\t",cdb->action);
  dbprintf("lock:%d\t",cdb->lock_number);
  dbprintf("version:%d\t",cdb->input_version_number);
  dbprintf("cid:%x\n",(int)cdb->cid);
}

void print_mode_cdb(mode_cdb *cdb){
  dbprintf("code:%d\t",cdb->code);
  dbprintf("max_clients:%d\t",cdb->max_clients);
  dbprintf("num_locks:%d\t",cdb->num_locks);
  dbprintf("lock_time:%d\n",cdb->lock_time_ms);
}



void print_dlock_rdf(dlock_rdf *rdf){
  int j;
  
  j=0;
  dbprintf("result:%d\t",rdf->result);
  dbprintf("act:%d\t",rdf->act);
  dbprintf("expired:%d\t",rdf->expired);
  dbprintf("state:%d\t",rdf->state);
  dbprintf("version:%d\t",rdf->version_number);
  dbprintf("holders:%d\n",rdf->num_holders);
  dbprintf("\n");
}


void print_rexpired_rdf(rexpired_rdf *rdf){
  int i,j;

  j=0;
  dbprintf("REX:");
  for(i=0; i<DLOCKS_PER_PARTITION/8; i++){
    dbprintf("%d:%d\t",i,rdf->exlock_block[i]);
    if(j++>5){
      dbprintf("\n");
    }
  }
  dbprintf("\n\n");
}

void print_mode_page_rdf(mode_page_rdf *rdf){
  dbprintf("MODE:");
  dbprintf("num_locks:%d\t",rdf->num_locks);
  dbprintf("max_clients:%d\t",rdf->max_clients);
  dbprintf("lock_time_ms:%d\n\n",rdf->lock_time_ms);
}

void print_drop_RDF(drop_rdf *rdf){
  dbprintf("DROP:lock_number:%d\t",rdf->lock_number);
  dbprintf("flags:%x\n\n",rdf->flags);
}


#endif
