/******************************************************************************
*******************************************************************************
**
**    Copyright 1995-1999 Regents of the University of Minnesota
**    Portions Copyright 1999 Sistina Software, Inc.  <kpreslan@sistina.com>
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Kenneth W. Preslan <kpreslan@lcse.umn.edu>
** 
**    Module name: gfshex.c
**
**    Description:
**	GFS-specific routines supportive of hexedit.
**
*******************************************************************************
******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <time.h>

#include "gfs_ondisk.h"
#include "hexedit.h"
#include "gfs_endian.h"
#include "gfshex.h"

/* Preprocessor Defines */


/* Function Macros */


/* Type Definitions */


/* Global Variables */


/******************************************************************************
*******************************************************************************
**
** int edit_gfs()
**
** Description:
**   This routine...
**
** Input(s):
**  *buffer   - 
**   extended - 
**
** Returns:
**   0 if OK, 1 on error.
**
*******************************************************************************
******************************************************************************/

int edit_gfs(char *buffer, int extended)
{
  int magic;

  magic = be32_to_cpup((int *)buffer);

  if (magic == GFS_MAGIC_SB)
    {
      edit_super(buffer, extended);
    }
  else if (magic == GFS_MAGIC_RG)
    {
      edit_rgrp(buffer, extended);
    }
  else if (magic == GFS_MAGIC_DI)
    {
      edit_dinode(buffer, extended);
    }
  else if (magic == GFS_MAGIC_LF)
    {
      edit_leaf(buffer, extended);
    }
  else
    {
      fprintf(stderr, "Bad magic number: 0x%08x\n\n", magic);
    }

  return(0);
}



/******************************************************************************
*******************************************************************************
**
** int edit_dinode()
**
** Description:
**   This routine takes in a buffer, which is a dinode, and edits the data
**	within.
**
** Input(s):
**  *buffer   - 
**   extended - 
**
** Returns:
**   0 if OK, 1 on error.
**
*******************************************************************************
******************************************************************************/

int edit_dinode(char *buffer, int extended){
  int option = 0;
  gfs_dinode_t di;


  printf("Dinode\n\n");
  printf("1  - Magic number                     12 - Major number\n");
  printf("2  - File system block address        13 - Minor number\n");
  printf("3  - Mode                             14 - Resource group for dinode\n");
  printf("4  - UID                              15 - Rgrp of last meta blk\n");
  printf("5  - GID                              16 - Block of last meta blk\n");
  printf("6  - Link count                       17 - Rgrp of last data blk\n");
  printf("7  - Size in bytes                    18 - Block of last data blk\n");
  printf("8  - Size in file system blocks       19 - Device lock number\n");
  printf("9  - Access time                      20 - Type\n");
  printf("10 - Modify time                      21 - Tree height\n");
  printf("11 - Change time                      22 - Stamp\n");
  printf("23 - Open count                       24 - Open bitmap\n");
  printf("\nEnter the field you wish to edit. [1-24] ");
  scanf(" %d", &option);

  gfs_dinode_in(&di, buffer);

  switch(option){
  case 1:
    printf("Not implemented\n");
    break;
  case 2:
    printf("Not implemented\n");
    break;
  case 3:
    printf("Not implemented\n");
    break;
  case 4:
    printf("Not implemented\n");
    break;
  case 5:
    printf("Not implemented\n");
    break;
  case 6:
    printf("Not implemented\n");
    break;
  case 7:
    printf("Not implemented\n");
    break;
  case 8:
    printf("Not implemented\n");
    break;
  case 9:
    printf("Not implemented\n");
    break;
  case 10:
    printf("Not implemented\n");
    break;
  case 11:
    printf("Not implemented\n");
    break;
  case 12:
    printf("Not implemented\n");
    break;
  case 13:
    printf("Not implemented\n");
    break;
  case 14:
    printf("Not implemented\n");
    break;
  case 15:
    printf("Not implemented\n");
    break;
  case 16:
    printf("Not implemented\n");
    break;
  case 17:
    printf("Not implemented\n");
    break;
  case 18:
    printf("Not implemented\n");
    break;
  case 19:
    printf("Not implemented\n");
    break;
  case 20:
    printf("Not implemented\n");
    break;
  case 21:
    printf("Not implemented\n");
    break;
  case 22:
    printf("Not implemented\n");
    break;
  case 23:
    printf("Not implemented\n");
    break;
  case 24:
    printf("Not implemented\n");
    break;
  default:
    printf("That is not a valid option\n");
    break;
  }
  if(extended)
    printf("Extended structure modification has not been implemented!\n");

  gfs_dinode_out(&di, buffer);
  return 1;
}

/******************************************************************************
*******************************************************************************
**
** int edit_rgrp()
**
** Description:
**   This routine takes in a buffer, which is a rgrp, and edits the data
**	within.
**
** Input(s):
**  *buffer   - 
**   extended - 
**
** Returns:
**   0 if OK, 1 on error.
**
*******************************************************************************
******************************************************************************/

int edit_rgrp(char *buffer, int extended){
  int option = 0;
  gfs_rgrp_t	 rg;

  printf("Resource Group\n\n");
  printf("1 - Magic number\n");
  printf("2 - Last Mod time\n");
  printf("3 - Free Blocks\n");
  printf("4 - Dinodes\n");
  printf("\nEnter the field you wish to edit. [1-4] ");
  scanf(" %d", &option);

  gfs_rgrp_in(&rg, buffer);

  switch(option){
  case 1:
    printf("Magic number (0x%x): 0x", rg.rg_magic);
    scanf(" %x", &rg.rg_magic);
    break;
  case 2:
    printf("Not implemented.\n");
    break;
  case 3:
    printf("Free Blocks (%"FMT_64"u): ", rg.rg_free);
    scanf(" %"FMT_64"u", &rg.rg_free);
    break;
  case 4:
    printf("Dinodes (%"FMT_64"u): ", rg.rg_dinodes);
    scanf(" %"FMT_64"u", &rg.rg_dinodes);
    break;
  default:
    printf("That is not a valid option\n");
    break;
  }
  if(extended)
    printf("Extended structure modification has not been implemented!\n");

  gfs_rgrp_out(&rg, buffer);

  return 1;
}

/******************************************************************************
*******************************************************************************
**
** int edit_super()
**
** Description:
**   This routine takes in a buffer, which is a super blk, and edits the data
**	within.
**
** Input(s):
**  *buffer   - 
**   extended - 
**
** Returns:
**   0 if OK, 1 on error.
**
*******************************************************************************
******************************************************************************/

int edit_super(char *buffer, int extended){
  int option = 0;
  gfs_sb_t	sb;

  printf("1  - Magic number                     11 - Lock Table Name\n");
  printf("2  - GFS Version                      12 - Sizeof SB structure\n");
  printf("3  - File system block size           13 - Sizeof RI structure\n");
  printf("4  - Length in FS blocks              14 - Sizeof RG structure\n");
  printf("5  - Root dinode address              15 - Sizeof DI structure\n");
  printf("6  - Number of resource groups        16 - Sizeof DE structure\n");
  printf("7  - File system Name                 17 - Sizeof LF structure\n");
  printf("8  - Number of blocks                 18 - Number of mounts\n");
  printf("9  - Number of device locks           19 - Mount ID bitmap\n");
  printf("10 - Lock Protocol Name               20 - Last Mod time\n");
  printf("\nEnter the field you wish to edit. [1-20] ");
  scanf(" %d", &option);

  gfs_sb_in(&sb, buffer);

  switch(option){
  case 1:
    printf("Magic number (0x%x): 0x", sb.sb_magic);
    scanf(" %x", &sb.sb_magic);
    break;
  case 2:
    printf("Not implemented.\n");
    break;
  case 3:
    printf("Not implemented.\n");
    break;
  case 4:
    printf("Not implemented.\n");
    break;
  case 5:
    printf("Root dinode address (%"FMT_64"u): ", sb.sb_rootdi);
    scanf(" %"FMT_64"u", &sb.sb_rootdi);
    break;
  case 6:
    printf("Not implemented.\n");
    break;
  case 7:
    printf("Not implemented.\n");
    break;
  case 8:
    printf("Not implemented.\n");
    break;
  case 9:
    printf("Not implemented.\n");
    break;
  case 10:
    printf("Not implemented.\n");
    break;
  case 11:
    printf("Not implemented.\n");
    break;
  case 12:
    printf("Not implemented.\n");
    break;
  case 13:
    printf("Not implemented.\n");
    break;
  case 14:
    printf("Not implemented.\n");
    break;
  case 15:
    printf("Not implemented.\n");
    break;
  case 16:
    printf("Not implemented.\n");
    break;
  case 17:
    printf("Not implemented.\n");
    break;
  case 18:
    printf("Not implemented.\n");
    break;
  case 19:
    printf("Not implemented.\n");
    break;
  case 20:
    printf("Not implemented.\n");
    break;
  default:
    printf("That is not a valid option\n");
    break;
  }
  if(extended)
    printf("Extended structure modification has not been implemented!\n");

  gfs_sb_out(&sb, buffer);
  return 1;
}

/******************************************************************************
*******************************************************************************
**
** int edit_leaf()
**
** Description:
**   This routine takes in a buffer, which is a leaf, and edits the data
**	within.
**
** Input(s):
**  *buffer   - 
**   extended - 
**
** Returns:
**   0 if OK, 1 on error.
**
*******************************************************************************
******************************************************************************/

int edit_leaf(char *buffer, int extended){
  int option = 0;
  gfs_leaf_t	lf;

  printf("Directory Leaf Block\n\n");
  printf("1 - Magic number\n");
  printf("2 - Number of entries\n");
  printf("3 - Depth\n");
  printf("4 - Next Leaf\n");

  gfs_leaf_in(&lf, buffer);

  switch(option){
  case 1:
    printf("Magic number (0x%x): ", lf.lf_magic);
    scanf(" %x", &lf.lf_magic);
    break;
  case 2:
    printf("Number of entries (%u): ", lf.lf_entries);
    scanf(" %hu", &lf.lf_entries);
    break;
  case 3:
    printf("Depth (%u): ", lf.lf_depth);
    scanf(" %hu", &lf.lf_depth);
    break;
  case 4:
    printf("Next Leaf (%"FMT_64"d): ", lf.lf_next);
    scanf(" %"FMT_64"d", &lf.lf_next);
    break;
  default:
    printf("That is not a valid option\n");
    break;
  }
  if(extended)
    printf("Extended structure modification has not been implemented!\n");

  gfs_leaf_out(&lf, buffer);
  return 1;
}

/******************************************************************************
*******************************************************************************
**
** int display_gfs()
**
** Description:
**   This routine...
**
** Input(s):
**  *buffer   - 
**   extended - 
**
** Returns:
**   0 if OK, 1 on error.
**
*******************************************************************************
******************************************************************************/

int display_gfs(char *buffer, int extended)
{
  int magic;

  magic = be32_to_cpup((int *)buffer);

  if (magic == GFS_MAGIC_SB)
    {
      print_super(buffer, extended);
    }
  else if (magic == GFS_MAGIC_RG)
    {
      print_rgrp(buffer, extended);
    }
  else if (magic == GFS_MAGIC_DI)
    {
      print_dinode(buffer, extended);
    }
  else if (magic == GFS_MAGIC_LF)
    {
      print_leaf(buffer, extended);
    }
  else
    {
      fprintf(stderr, "Bad magic number: 0x%08x\n\n", magic);
    }

  return(0);
}


/******************************************************************************
*******************************************************************************
**
** int print_dinode()
**
** Description:
**   This routine...
**
** Input(s):
**  *buffer   - 
**   extended - 
**
** Returns:
**
**
*******************************************************************************
******************************************************************************/

int print_dinode(char *buffer, int extended)
{
  gfs_dinode_t di;
  time_t ti;
  int i;

  gfs_dinode_in(&di, buffer);

  printf("Dinode\n\n");

  printf("Magic number:                0x%.8X\n", di.di_magic);

  printf("File system block address:   %"FMT_64"u\n", di.di_addr);

  printf("Mode:                        0%o\n", di.di_mode);
  printf("UID:                         %u\n", di.di_uid);
  printf("GID:                         %u\n", di.di_gid);
  printf("Link count:                  %u\n", di.di_nlink);
  printf("Size in bytes:               %"FMT_64"d\n", di.di_bsize);
  printf("Size in file system blocks:  %"FMT_64"d\n", di.di_blocks);
  ti = (long)di.di_atime;
  printf("Access time:                 %s", ctime(&ti));
  ti = (long)di.di_mtime;
  printf("Modify time:                 %s", ctime(&ti));
  ti = (long)di.di_ctime;
  printf("Change time:                 %s", ctime(&ti));
  printf("Major number:                %u\n", di.di_major);
  printf("Minor number:                %u\n", di.di_minor);

  printf("Resource group for dinode:   %u\n", di.di_rgrp);
  printf("Rgrp of last meta blk:       %u\n", di.di_rgrp_meta);
  printf("Block of last meta blk:      %"FMT_64"u\n", di.di_last_meta);
  printf("Rgrp of last data blk:       %u\n", di.di_rgrp_data);
  printf("Block of last data blk:      %"FMT_64"u\n", di.di_last_data);
  printf("Device lock number:          %"FMT_64"u\n", di.di_glock);
  printf("Type:                        %u ", di.di_type);
  switch(di.di_type)
    {
    case FILE_REG:
      printf("(Regular file)\n");
      break;
    case FILE_DIR:
      printf("(Directory)\n");
      break;
    case FILE_LNK:
      printf("(Symlink)\n");
      break;
    case FILE_BLK:
      printf("(Block device)\n");
      break;
    case FILE_CHR:
      printf("(Character device)\n");
      break;
    case FILE_FIFO:
      printf("(FIFO)\n");
      break;
    case FILE_NON:
    default:
      printf("(Bad file)\n");
      break;
    }
  printf("Tree height:                 %d\n", di.di_height);
  printf("Stamp:                       %d\n", di.di_stamp);

  printf("Open count:                  %u\n", di.di_nopen);
  printf("Open bitmap:                 ");
  for (i = 0; i < GFS_MOUNT_BITS; i++)
    printf("%.2X ", di.di_open[i]);
  printf("\n");

  if (di.di_type == 2)
    {
      printf("Regime:                      %d (%s)\n", di.di_regime,
	     (di.di_regime == GFS_DIR_LINEAR) ? "Linear" : "ExHash");
      printf("Depth:                       %d\n", di.di_depth);
      printf("Number of Entries            %d\n", di.di_entries);
    }

  if ((extended == 0) || (di.di_bsize == 0)) 
    return (0);

  printf("No Extended Data  (FixMe!!!)\n");

  return(0);
}


/******************************************************************************
*******************************************************************************
**
** int print_rgrp()
**
** Description:
**   This routine...
**
** Input(s):
**  *buffer   - 
**   extended - 
**
** Returns:
**
**
*******************************************************************************
******************************************************************************/

int print_rgrp(char *buffer, int extended)
{
  gfs_rgrp_t	 rg;
  time_t ti;

  gfs_rgrp_in(&rg, buffer);

  printf("Resource Group\n\n");
  
  printf("Magic number:                      0x%x\n", rg.rg_magic);

  ti = (long)rg.rg_time;
  printf("Last Mod time                      %s", ctime(&ti));

  printf("Free Blocks:                       %"FMT_64"u\n", rg.rg_free);
  printf("Dinodes:                           %"FMT_64"u\n", rg.rg_dinodes);

  if (extended == 0)
    return (0);

  printf("No Extended Data  (FixMe!!!)\n");

  return(0);
}


/******************************************************************************
*******************************************************************************
**
** int print_super()
**
** Description:
**   This routine...
**
** Input(s):
**  *buffer   - 
**   extended - 
**
** Returns:
**
**
*******************************************************************************
******************************************************************************/

int print_super(char *buffer, int extended)
{
  int           i;
  gfs_sb_t	sb;
  /*  gfs_rindex_t  *ri, *rip;*/
  char          name[256];
  time_t        ti;


  gfs_sb_in(&sb, buffer);

  printf("Super Block\n\n");
  
  printf("Magic number:                 0x%x\n", sb.sb_magic);
  printf("GFS Version:                  %0.8X\n", sb.sb_version);

  printf("File system block size:       %u\n", sb.sb_bsize);
  printf("Length in FS blocks:          %u\n", sb.sb_length);

  printf("Root dinode address:          %"FMT_64"u\n", sb.sb_rootdi);
  printf("Number of resource groups:    %u\n", sb.sb_rgrps);

  memset(name, 0, 256);
  memcpy(name, sb.sb_name, 12);
  printf("File system Name:             %s\n", name);

  printf("Number of blocks:             %"FMT_64"u\n", sb.sb_fssize);

  printf("Number of device locks:       %"FMT_64"u\n", sb.sb_glocks);
  printf("Lock Protocol Name:           %s\n", sb.sb_lockproto);
  printf("Lock Table Name:              %s\n", sb.sb_locktable);

  printf("Sizeof SB structure:          %u\n", sb.sb_sbsize);
  printf("Sizeof RI structure:          %u\n", sb.sb_risize);
  printf("Sizeof RG structure:          %u\n", sb.sb_rgsize);
  printf("Sizeof DI structure:          %u\n", sb.sb_disize);
  printf("Sizeof DE structure:          %u\n", sb.sb_desize);
  printf("Sizeof LF structure:          %u\n", sb.sb_lfsize);

  printf("Number of mounts:             %u\n", sb.sb_mounts);
  printf("Mount ID bitmap:              ");
  for (i = 0; i < GFS_MOUNT_BITS; i++)
    printf("%.2X ", sb.sb_mountids[i]);
  printf("\n");
  ti = (long)sb.sb_time;
  printf("Last Mod time:                %s", ctime(&ti));
  

  if (!extended)
    return(0);

  /*
  ri = (gfs_rindex_t *)(buffer + sizeof(gfs_sb_t));
  for (i = 0; i < sb.sb_rgrps; i++)
    {
      rip = &ri[i];

      printf("\nResource Index #%d\n\n", i);

      printf("Address of RGRP header:       %"FMT_64"u\n", rip->ri_addr);
      printf("Length of RGRP header:        %u\n", rip->ri_length);
      printf("Dlock number:                 %"FMT_64"u\n", rip->ri_glock);
      printf("SubPool:                      %u\n", rip->ri_subpool);

      printf("First data block:             %"FMT_64"u\n", rip->ri_data1);
      printf("Number of data blocks:        %"FMT_64"u\n", rip->ri_data);

      printf("First dlock:                  %"FMT_64"u\n", rip->ri_glock1);
      printf("Number of dlocks:             %"FMT_64"u\n", rip->ri_glocks);

      printf("Length of bitmaps:            %u\n", rip->ri_bitbytes);
    }
    */
  printf("No Extended Data  (FixMe!!!)\n");

  return(0);
}


/******************************************************************************
*******************************************************************************
**
** int print_leaf()
**
** Description:
**   This routine...
**
** Input(s):
**  *buffer   - 
**   extended - 
**
** Returns:
**
**
*******************************************************************************
******************************************************************************/

int print_leaf(char *buffer, int extended)
{
  gfs_leaf_t	lf;

  gfs_leaf_in(&lf, buffer);

  printf("Directory Leaf Block\n\n");
  
  printf("Magic number:                      0x%x\n", lf.lf_magic);
  printf("Number of entries:                 %d\n", lf.lf_entries);
  printf("Depth:                             %d\n", lf.lf_depth);
  printf("Next Leaf:                         %"FMT_64"d\n", lf.lf_next);

  if (extended == 0)
    return (0);

  printf("No Extended Data  (FixMe!!!)\n");

  return(0);
}
