/*-
 * Copyright (c) 1993, 1994
 *	The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static char copyright[] =
"@(#) Copyright (c) 1993, 1994\n\
	The Regents of the University of California.  All rights reserved.\n";
#endif /* not lint */

#ifndef lint
/*
static char sccsid[] = "@(#)mount_lfs.c	8.3 (Berkeley) 3/27/94";
*/
static const char rcsid[] =
	"$Id: mount_gfs.c,v 1.2 1999/07/09 00:50:42 kpreslan Exp $";
#endif /* not lint */

#include <sys/param.h>
#include <sys/mount.h>

#include <err.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sysexits.h>
#include <unistd.h>

/* #include <ufs/ufs/ufsmount.h> */
#include <lgfs.h>

#include "mntopts.h"

struct mntopt mopts[] = {
	MOPT_STDOPTS,
	MOPT_FORCE,
	MOPT_SYNC,
	MOPT_UPDATE,
	{ NULL }
};

static void	usage __P((void)) __dead2;

int
main( int argc,
	  char *argv[]) {
  /*  struct ufs_args args; */
  gfs_args_t args;
  int ch, mntflags;
  char *fs_name, *options;
  struct vfsconf vfc;
  int error;
  
  options = NULL;
  mntflags = 0;
  while ((ch = getopt(argc, argv, "o:")) != -1)
	switch (ch) {
#if 0
	case 'o':
	  getmntopts(optarg, mopts, &mntflags, 0);
	  break;
#endif
	case '?':
	default:
	  usage();
	}
  argc -= optind;
  argv += optind;
  
  if (argc != 2)
	usage();
  
  args.target = argv[0];	/* the name of the device file */
  strcpy(args.lockproto, "nolock");
  strcpy(args.locktable, "moo");
  strcpy(args.hostdata, "ack");
  fs_name = argv[1];	/* the mount point */
  
#define DEFAULT_ROOTUID	-2
#if 0
  args.export.ex_root = DEFAULT_ROOTUID;
  if (mntflags & MNT_RDONLY)
	args.export.ex_flags = MNT_EXRDONLY;
  else
	args.export.ex_flags = 0;
#endif
  
  error = getvfsbyname("gfs", &vfc);
  if (error && vfsisloadable("gfs")) {
	if (vfsload("gfs")) {
	  err(EX_OSERR, "vfsload(gfs)");
	}
	endvfsent();	/* flush cache */
	error = getvfsbyname("gfs", &vfc);
  }
  if (error)
	errx(EX_OSERR, "gfs filesystem is not available");
  
  if (mount(vfc.vfc_name, fs_name, mntflags, &args) < 0)
	err(EX_OSERR, "%s", args.target);
  exit(0);
}

void
usage() {
  (void)fprintf(stderr,
				"usage: mount_gfs [-o options] special node\n");
  exit(EX_USAGE);
}
