/******************************************************************************
*******************************************************************************
**
**    Copyright 1995-1999 Regents of the University of Minnesota
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      David C. Teigland <teigland@lcse.umn.edu>
** 
**    Module name: ptool.h
**
**    Description:
**	Creates, prints, and syncs labels for the storage pool device driver.
**      Gets pool driver statics (e.g. read/write counts, lock conflicts, etc)
**
*******************************************************************************
******************************************************************************/

#ifndef __PTOOL_H__
#define __PTOOL_H__

#include <asm/spinlock.h>

#include "pool.h"


#define BBSIZE          (1<<BBSHIFT)
#define BTOBB(bytes)    (((unsigned long)(bytes) + BBSIZE - 1) >> BBSHIFT)
#define LINE_MAX	256

#define OPTION_STRING	"hqV"



typedef struct new_dev_s {
  dev_t		nd_dev;
  uint32	nd_blocks;
  uint32	nd_dlocks;
  uint32	nd_dlonly;
} new_dev_t;

typedef struct new_spool_s {
  uint32	ns_blk_devs;
  uint32	ns_dlk_devs;
  uint32	ns_num_devs;
  uint32	ns_stripe;
  new_dev_t	*ns_devs;
} new_spool_t;

typedef struct new_pool_s {
  char		np_name[POOL_NAME_SIZE];
  uint32	np_num_spools;
  new_spool_t	*np_spools;
} new_pool_t;


typedef struct path_buf_s {
  dev_t      	dev;
  char		path[POOL_NAME_SIZE];
} path_buf_t;


/* Function Prototypes */

void			 decodeOptions(int argc, char *const argv[]);
void			 printUsage(int status);
void			 printVersion(void);
new_pool_t     		*getPoolParams(const char *paramFile);
void 			 parseError(const char *fileName, int lineNum, 
				    char *lineBuf, char *errMsg);
static char 		*get_path(dev_t dev);
int			 printPool(new_pool_t *pool);
int			 writeLabels(new_pool_t *pool);
int 			 getDeviceID(const char *specFile, dev_t *deviceID);

extern void pool_label_out(pool_label_t *pl, char *buf);

#endif /* __PTOOL_H__ */





