/******************************************************************************
*******************************************************************************
**
**    Copyright 1995-1999 Regents of the University of Minnesota
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Kenneth W. Preslan <kpreslan@lcse.umn.edu>
** 
**    Module name: global.h
**
**    Description:
**      Globally used header for GFS, pool, etc.
**
*******************************************************************************
******************************************************************************/

#include <stdio.h>

#ifndef __GLOBAL_DOT_H__
#define __GLOBAL_DOT_H__

#ifdef __cplusplus
extern "C" {
#endif

/* Preprocessor Defines */

#ifndef ULONGLONG_MAX
#define ULONGLONG_MAX       18446744073709551615LLU
#endif

#ifndef BPKB
#define	BPKB	10	/* Bits per kilobyte */
#define KBSHIFT	BPKB
#endif

#if !defined(KILOBYTE)
#define KILOBYTE	((__uint32_t)(       1 << KBSHIFT))
#endif

#if !defined(MEGABYTE)
#define MEGABYTE	((__uint32_t)(KILOBYTE << KBSHIFT))
#endif

#if !defined(GIGABYTE)
#define GIGABYTE	((__uint32_t)(MEGABYTE << KBSHIFT))
#endif

#if !defined(TERABYTE)
#define TERABYTE	(1099511627776LLU)
#endif

#if !defined(PETABYTE)
#define PETABYTE	(1125899906842624LLU)
#endif

#if !defined(TRUE) || ((TRUE) != 1)
#define TRUE	1
#endif

#if !defined(FALSE) || ((FALSE) != 0)
#define FALSE	0
#endif


/* Function Macros */

/* Type Definitions */

/* GFS Base Data Types */

#ifndef GFS_TYPES
#define GFS_TYPES



#ifdef linux

#ifdef __alpha__
#define FMT_64 "l"
#define FMT_32 ""
#endif  /*  __alpha__  */
#ifdef __i386__
#define FMT_64 "ll"
#define FMT_32 ""
#endif  /*  __i386__  */

#ifdef __KERNEL__
#include <linux/types.h>
#else
#include <sys/types.h>
typedef u_int64_t         uint64_t;
typedef u_int32_t         uint32_t;
typedef u_int16_t         uint16_t;
typedef u_int8_t          uint8_t;
#endif  /*  __KERNEL__  */

typedef int		minor_t;

#endif  /*  linux  */



#ifdef IRIX

#define FMT_64 "l"
#define FMT_32 ""

#include <sys/types.h>
typedef dev_t		kdev_t;

#endif  /*  IRIX  */



#if defined(__FreeBSD__)
#include <sys/types.h>

#define FMT_64 "ll"
#define FMT_32 ""

#endif  /*  __FreeBSD  */



typedef uint64_t          uint64;
typedef uint32_t          uint32;
typedef uint16_t          uint16;
typedef uint8_t           uint8;
typedef int64_t           int64;
typedef int32_t           int32;
typedef int16_t           int16;
typedef int8_t            int8;


#endif  /*  GFS_TYPES  */


typedef struct debugFlag_s
{
  char          *name;
  uint64         flag;
} debugFlag_t;


/* Global Variables */


/* Function Prototypes */


#ifdef __cplusplus
}
#endif

extern uint64 dl_flags;

#define DLF_DPRINTF     1
#define DLF_FUNCTIONS   2  
#define DLF_ERROR       4

#define dprintf(fmt, args...) {if (dl_flags & DLF_DPRINTF) printf(fmt, ## args);}
#define eprintf(d_level, fmt, args...) {if (dl_flags & d_level) printf(fmt, ## args);}





#endif /* __GLOBAL_H__ */

