#include <unistd.h>

#include "state.h"
#include "structs.h"
#include "misc.h"

#include "global.h"
#include "transport_list.h"
#include "dlock_interface.h"
#include "wrappers.h"
#include "suite_debug.h"

#define DO_DLOCK_FAIL(handle, data) (do_dlock(handle, data) || ! (data)->result)

int goto_state(device_handle_t *handle, dlock_state_t *st, uint32 lock, uint32 clientid) {
  dlock_data_t data;
  dlock_info_t info;
  uint32 old_timeout = 0;

  data.action = NOP;
  data.locknum = lock;
  data.clientid = clientid;

  
  ENTER(GOTO_STATE);
  
  
  if (DO_DLOCK_FAIL(handle, &data))
    die("goto_state: can't NOP\n");
  

  while (data.holders) {
    data.action = UNLOCK;
    data.clientid = data.clientids[0];

    if (DO_DLOCK_FAIL(handle, &data))
      die("goto_state: can't UNLOCK\n");
  }



  if (data.state != UNLOCKED)
    die("goto_state: not UNLOCKED\n");

  if (st->state == UNLOCKED) {
    EXIT(GOTO_STATE);
    return 0;
  }

  if (st->excl_pending && st->expired) {
    EXIT(GOTO_STATE);
    return 1;
  }


  if (st->expired) {
    get_info(handle, &info);
    old_timeout = info.timeout;
    info.timeout = 1;
    set_info(handle, &info);
  }


  switch (st->state) {
  case LOCKED_SHARED: data.action = LOCK_SHARED; break;
  case LOCKED_EXCL:   data.action = LOCK_EXCL; break;
  default: goto timeout_fail;
  }

  data.clientid = clientid;
  
  if (DO_DLOCK_FAIL(handle, &data))
    goto timeout_fail;

  if (st->expired) {
    sleep(1);
    data.action = NOP;

    if (DO_DLOCK_FAIL(handle, &data))
      goto timeout_fail;

    if (!data.expired)
      goto timeout_fail;

    goto timeout_success;
  }


  if (st->excl_pending) {
    data.action = LOCK_EXCL;
    
    if (DO_DLOCK_FAIL(handle, &data))
      goto timeout_fail;

    if (data.result)
      goto timeout_fail;

    goto timeout_success;
  }


 timeout_success:

  if (st->expired) {
    info.timeout = old_timeout;
    set_info(handle, &info);
  }


  EXIT(GOTO_STATE);
  return 0;

 timeout_fail:

  if (st->expired) {
    info.timeout = old_timeout;
    set_info(handle, &info);
  }

  EXIT(GOTO_STATE);
  return 1;
  
}
  

