#include <stdlib.h>

#include "dlock_interface.h"
#include "transport_list.h"
#include "global.h"
#include "trans_debug.h"

/* name = type of transport
 * table_name = name of instance of that transport
 */

device_handle_t * dl_open(char *name, char *table_name) {
  int x;
  device_handle_t *dev;
  dlock_interface_t *t = NULL;

  ENTER(DL_OPEN);

  if ((dev = malloc(sizeof(*dev))) == NULL) {
    eprintf(DLF_ERROR, "dl_open: can't malloc for dev\n");

    EXIT(DL_OPEN);
    return NULL;
  }

  for (x = 0; transports[x]; x++) {
    
    if (!strcmp(transports[x]->name, name)) {
      t = transports[x];
      dprintf("dl_open: using transport %s\n", t->name);
      /*break;*/
    }

    if (t != transports[x])
      dprintf("dl_open: skipping transport %s\n", transports[x]->name);
  }

  dprintf("dl_open: %d transports registered\n", x);

  if (!t) {
    eprintf(DLF_ERROR, "dl_open: transport not found\n");
    goto free_handle;
  }

  dev->t = t;

  dev->name = dev->t->open(table_name);

  if (!dev->name) {
    eprintf(DLF_ERROR, "dl_open: transport specific open failed\n");
    goto free_handle;
  }

  EXIT(DL_OPEN);
  return dev;

free_handle:
  free(dev);
  EXIT(DL_OPEN);
  return NULL;

}

void dl_close(device_handle_t *dev) {
  ENTER(DL_CLOSE);
  
  dev->t->close(dev->name);
  
  EXIT(DL_CLOSE);
  free(dev);
}

int do_dlock(device_handle_t *dev, dlock_data_t *data) {
  int retval;

  ENTER(DO_DLOCK);

  if (!dev->t) {
    eprintf(DLF_ERROR, "do_dlock: no transport for handle\n");
    EXIT(DO_DLOCK);
    return 1;
  }

  retval =  dev->t->do_dlock(dev->name, data);

  EXIT(DO_DLOCK);
  return retval;
}


int get_info(device_handle_t *dev, dlock_info_t *info) {
  ENTER(GET_INFO);
  
  if (!dev->t) {
    eprintf(DLF_ERROR, "get_info: not transport for handle\n");
    EXIT(GET_INFO);
    return 1;
  }
  
  EXIT(GET_INFO);
  return dev->t->get_info(dev->name, info);
}

int set_info(device_handle_t *dev, dlock_info_t *info) {
  ENTER(SET_INFO);

  if (!dev->t) {
    eprintf(DLF_ERROR, "set_info: no transport for handle\n");
    EXIT(SET_INFO);
    return 1;
  }

  EXIT(SET_INFO);
  return dev->t->set_info(dev->name, info);
}
